/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#include "anchor.h"
#include "tcpsocket.h"

/*!
    \class Anchor
    \brief La classe Anchor est le pont entre le QML et le C++.

    La classe Anchor permet la commnication entre le QML et le C++ du fait
    qu'une instance définie dans la classe MainWindow puisse à la fois être
    manipulée dans le C++ en tant que simple instance et dans le QML en tant
    que propriété contextuelle.
*/

/*!
    \property Anchor::latency
    \brief La latence de la connexion au serveur.
*/

/*!
    \property Anchor::connected
    \brief L'état de la connexion de l'utilisateur.
*/

/*!
    \property Anchor::username
    \brief Le nom de l'utilisateur.
*/

/*!
    \fn Anchor::Anchor(QObject*)
    \brief Constructeur de la classe.

    Construit un objet Anchor.
*/

Anchor::Anchor(QObject*) : _socket(NULL), _latency(0), _connected(false) { }

/*!
    \fn Anchor::setSocket(TcpSocket *socket)
    \brief Permet à la classe Anchor d'accéder à l'instance \a socket de la
    classe TcpSocket.
*/

void Anchor::setSocket(TcpSocket *socket) {
    _socket = socket;
}

/*!
    \fn Anchor::setLatency(const int &latency)
    \brief Définit la latence de la connexion entre le client et le serveur.
*/

void Anchor::setLatency(const int &latency) {
    _latency = latency;
    emit latencyChanged();
}

int Anchor::latency() const {
    return _latency;
}

/*!
    \fn Anchor::setConnected(const bool &connected)
    \brief Définit si le client est connecté au serveur.
*/

void Anchor::setConnected(const bool &connected) {
    _connected = connected;
    emit connectionChanged();
}

bool Anchor::connected() const {
    return _connected;
}

/*!
    \fn Anchor::setUsername(const QString &username)
    \brief Définit le nom de l'utilisateur du client.
*/

void Anchor::setUsername(const QString &username) {
    _username = username;
    emit usernameChanged();
}

QString Anchor::username() const {
    return _username;
}

/*!
    \fn Anchor::login(QString username, QString password)
    \brief Appelle la fonction sendLogin() du socket.
*/

void Anchor::login(QString username, QString password) {
    SOCKET->sendLogin(username, password);
}

/*!
    \fn Anchor::requestForMedicineModel()
    \brief Appelle la fonction sendRequestForMedicineModel() du socket.
*/

void Anchor::requestForMedicineModel() {
    SOCKET->sendRequestForMedicineModel();
}

/*!
    \fn Anchor::requestForPatientsModel()
    \brief Appelle la fonction requestForPatientsModel() du socket.
*/

void Anchor::requestForPatientsModel() {
    SOCKET->sendRequestForPatients();
}

/*!
    \fn Anchor::requestForBodyPartsModel()
    \brief Appelle la fonction sendRequestForBodyParts() du socket.
*/

void Anchor::requestForBodyPartsModel() {
    SOCKET->sendRequestForBodyParts();
}

/*!
    \fn Anchor::requestForPatient(int id)
    \brief Appelle la fonction requestForPatient() du socket.
*/

void Anchor::requestForPatient(int id) {
    SOCKET->sendRequestForPatient(id);
}

/*!
    \fn Anchor::modifyPatient(int id, QVariant data)
    \brief Appelle la fonction sendPatientInfos() du socket.
*/

void Anchor::modifyPatient(int id, QVariant data) {
    QStringList list = data.toStringList();
    if (!list.isEmpty())
        SOCKET->sendPatientInfos(id, list);
}

/*!
    \fn Anchor::addPatient(QVariant data)
    \brief Appelle la fonction sendPatientInfos() du socket avec l'id de
    patient -1 pour informer qu'il s'agit d'un ajout de patient.
*/

void Anchor::addPatient(QVariant data) {
    QStringList list = data.toStringList();
    if (!list.isEmpty())
        SOCKET->sendPatientInfos(-1, list);
}

/*!
    \fn Anchor::newDiagnostic(int id, QString data)
    \brief Appelle la fonction sendNewDiagnostic() du socket.
*/

void Anchor::newDiagnostic(int id, QString data) {
    SOCKET->sendNewDiagnostic(id, data);
}

/*!
    \fn Anchor::newSymptom(int id, QString data)
    \brief Appelle la fonction sendNewSymptom() du socket.
*/

void Anchor::newSymptom(int id, QString data) {
    SOCKET->sendNewSymptom(id, data);
}

/*!
    \fn Anchor::removePrescription(int patientId, int id)
    \brief Appelle la fonction sendRemovePrescription() du socket.
*/

void Anchor::removePrescription(int patientId, int id) {
    SOCKET->sendRemovePrescription(patientId, id);
}

/*!
    \fn Anchor::newPrescription(int patientId, QString duration, int date, int medicineId)
    \brief Appelle la fonction sendNewPrescription() du socket avec l'id de
    prescription -1 pour informer qu'il s'agit d'un ajout de prescription.
*/

void Anchor::newPrescription(int patientId, QString duration, int date, int medicineId) {
    SOCKET->sendPrescription(patientId, -1, duration, date, medicineId);
}

/*!
    \fn Anchor::modifyPrescription(int patientId, int id, QString duration, int date, int medicineId)
    \brief Appelle la fonction sendModifyPrescription() du socket.
*/

void Anchor::modifyPrescription(int patientId, int id, QString duration, int date, int medicineId) {
    SOCKET->sendPrescription(patientId, id, duration, date, medicineId);
}

/*!
    \fn Anchor::enterChat()
    \brief Appelle la fonction sendEnterChat() du socket.
*/

void Anchor::enterChat() {
    SOCKET->sendEnterChat();
}

/*!
    \fn Anchor::leaveChat()
    \brief Appelle la fonction sendLeaveChat() du socket.
*/

void Anchor::leaveChat() {
    SOCKET->sendLeaveChat();
}

/*!
    \fn Anchor::sendMessageInChat(QString message)
    \brief Appelle la fonction sendMessageInChat() du socket.
*/

void Anchor::sendMessageInChat(QString message) {
    SOCKET->sendMessageInChat(message);
}

/*!
    \fn Anchor::getChatUsers()
    \brief Appelle la fonction sendChatUsersRequest() du socket.
*/

void Anchor::getChatUsers() {
    SOCKET->sendChatUsersRequest();
}

/*!
    \fn Anchor::startECG(int index)
    \brief Appelle la fonction sendStartECG() du socket avec l'id \a index du
    patient concerné.
*/

void Anchor::startECG(int index) {
    SOCKET->sendStartECG(index);
}

/*!
    \fn Anchor::stopECG()
    \brief Appelle la fonction sendStopECG() du socket.
*/

void Anchor::stopECG() {
    SOCKET->sendStopECG();
}

/*!
    \fn Anchor::requestForAnalysisModel()
    \brief Appelle la fonction sendRequestForAnalysis() du socket.
*/

void Anchor::requestForAnalysisModel() {
    SOCKET->sendRequestForAnalysis();
}

/*!
    \fn Anchor::requestForScan(int id, int part)
    \brief Requête de le la partie \a part du scan à l'id \a id.

    Un système de cache en local à été mis en place afin d'éviter de trop
    carger la bande-passante entre le client et le serveur lors du transfert
    des images. Ainsi, si l'image est déjà présente en cache, on n'envoit pas
    de requête au serveur et on retourne directement le chemin vers l'image.
    Le cas échéant, appelle la fonction sendRequestForScan() du socket.
*/

void Anchor::requestForScan(int id, int part) {
    QString path = qApp->applicationDirPath() + "/scans/scan_"
        + QString::number(id) + "_" + QString::number(part) + ".jpg";
    if (QFileInfo(path).exists()) {
        receiveScan(id, part, path);
        return;
    }
    SOCKET->sendRequestForScan(id, part);
}

/*!
    \fn Anchor::requestForRadio(int id, int x, int y)
    \brief Requête de le la partie (\a x, \a y) de la radio à l'id \a id.

    Un système de cache en local à été mis en place afin d'éviter de trop
    carger la bande-passante entre le client et le serveur lors du transfert
    des images. Ainsi, si l'image est déjà présente en cache, on n'envoit pas
    de requête au serveur et on retourne directement le chemin vers l'image.
    Le cas échéant, appelle la fonction sendRequestForRadio() du socket.
*/

void Anchor::requestForRadio(int id, int x, int y) {
    QString path = qApp->applicationDirPath() + "/radios/radio_"
        + QString::number(id) + "_" + QString::number(x) + "_"
        + QString::number(y) + ".jpg";
    if (QFileInfo(path).exists()) {
        receiveRadio(id, x, y, path);
        return;
    }
    SOCKET->sendRequestForRadio(id, x, y);
}
