/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#ifndef ANCHOR_H
#define ANCHOR_H

#include <QObject>
#include <QVariant>
#include <QDebug>

class TcpSocket;
class PatientModel;

class Anchor : public QObject
{
    Q_OBJECT
    Q_PROPERTY(int latency READ latency WRITE setLatency NOTIFY latencyChanged)
    Q_PROPERTY(bool connected READ connected WRITE setConnected NOTIFY connectionChanged)
    Q_PROPERTY(QString username READ username WRITE setUsername NOTIFY usernameChanged)

public:
    Anchor(QObject *parent = 0);
    void setSocket(TcpSocket*);

    // QML
    Q_INVOKABLE void login(QString, QString);
    Q_INVOKABLE void requestForMedicineModel();
    Q_INVOKABLE void requestForPatientsModel();
    Q_INVOKABLE void requestForBodyPartsModel();
    Q_INVOKABLE void requestForPatient(int);
    Q_INVOKABLE void modifyPatient(int, QVariant);
    Q_INVOKABLE void addPatient(QVariant);
    Q_INVOKABLE void newDiagnostic(int, QString);
    Q_INVOKABLE void newSymptom(int, QString);
    Q_INVOKABLE void removePrescription(int, int);
    Q_INVOKABLE void newPrescription(int, QString, int, int);
    Q_INVOKABLE void modifyPrescription(int, int, QString, int, int);
    Q_INVOKABLE void enterChat();
    Q_INVOKABLE void leaveChat();
    Q_INVOKABLE void sendMessageInChat(QString);
    Q_INVOKABLE void getChatUsers();
    Q_INVOKABLE void startECG(int);
    Q_INVOKABLE void stopECG();
    Q_INVOKABLE void requestForAnalysisModel();
    Q_INVOKABLE void requestForScan(int, int);
    Q_INVOKABLE void requestForRadio(int, int, int);

    // GlobalHooks
    /*! Retourne la latence de la connexion entre le client et le serveur. */
    int latency() const;
    void setLatency(const int&);
    /*! Appelé lorsque la latence de la connexion au serveur change. */
    Q_SIGNAL void latencyChanged();
    /*! Retourne si le client est connecté au serveur. */
    bool connected() const;
    void setConnected(const bool&);
    /*! Appelé lorsque l'état de la connexion au serveur change. */
    Q_SIGNAL void connectionChanged();
    /*! Appelé lorsqu'une nouvelle tentative de connexion a lieu. */
    Q_SIGNAL void reconnectTry(int number);

    // AccountHooks
    /*! Retourne le nom de l'utilisateur du client. */
    QString username() const;
    void setUsername(const QString&);
    /*! Appelé lorsque le nom de l'utilisateur du client change. */
    Q_SIGNAL void usernameChanged();
    /*! Appelé lorsqu'une tentative d'authentification a réussi. */
    Q_SIGNAL void loginSuccess();
    /*! Appelé lorsqu'une tentative d'authentification a échoué. */
    Q_SIGNAL void loginFailure();

    // ModuleHooks
    /*! Appelé lors de la réception du modèle des médicaments. */
    Q_SIGNAL void receiveMedicineModel(const QString &medicine);
    /*! Appelé lors de la réception du modèle des patients. */
    Q_SIGNAL void receivePatientsModel(const QString &patients);
    /*! Appelé lors de la réception du modèle des parties du corps. */
    Q_SIGNAL void receiveBodyPartsModel(const QString &parts);
    /*! Appelé lors de la réception des informations d'un patient. */
    Q_SIGNAL void receivePatientInfos(const QString &patientInfos);
    /*! Appelé lors de la réception des informations d'un nouveau patient. */
    Q_SIGNAL void receiveNewPatientInfos(const QString &patientInfos);
    /*! Appelé suite à la réception des modifications d'un patient. */
    Q_SIGNAL void receivePatientModifs(int id, const QString &patientInfos);
    /*! Appelé suite à une notification d'ajout d'un diagnostic. */
    Q_SIGNAL void receiveNewDiagnostic(int id, const QString &diagnostic);
    /*! Appelé suite à une notification d'ajout d'un symptôme. */
    Q_SIGNAL void receiveNewSymptom(int id, const QString &symptom);
    /*! Appelé suite à une notification de suppression de prescription. */
    Q_SIGNAL void receiveRemovePrescription(int patientId, int id);
    /*! Appelé suite à une notification d'ajout de prescription. */
    Q_SIGNAL void receiveNewPrescription(int patientId);
    /*! Appelé suite à une notification de modification de prescription. */
    Q_SIGNAL void receiveModifyPrescription(int patientId);
    /*! Appelé suite à l'entrée d'un utilisateur dans le salon. */
    Q_SIGNAL void receiveEnterChat(const QString &username);
    /*! Appelé suite à la sortie d'un utilisateur dans le salon. */
    Q_SIGNAL void receiveLeaveChat(const QString &username);
    /*! Appelé suite à la réception du message d'un utilisateur dans le salon. */
    Q_SIGNAL void receiveChatMessage(const QString &username, const QString &message);
    /*! Appelé suite à la réception des utilisateurs présents dans le salon. */
    Q_SIGNAL void receiveChatUsers(const QStringList &users);
    /*! Appelé suite à la réception de points de l'ECG. */
    Q_SIGNAL void receiveEcgData(const QStringList &data);
    /*! Appelé suite à la réception du modèle des résultats d'analyses. */
    Q_SIGNAL void receiveAnalysisModel(const QString &analysis);
    /*! Appelé suite à la réception de l'image d'un scan. */
    Q_SIGNAL void receiveScan(int id, int part, const QString &path);
    /*! Appelé suite à la réception de l'image d'une radio. */
    Q_SIGNAL void receiveRadio(int id, int x_part, int y_part, const QString &path);

private:
    /*! L'instance de la classe TcpSocket qui commnique avec le serveur. */
    TcpSocket *_socket;
    /*! La latence de la connexion au serveur. */
    int _latency;
    /*! Définit si l'utilisateur est connecté ou non. */
    bool _connected;
    /*! Nom de l'utilisateur. */
    QString _username;
};

#define SOCKET \
    if (_socket) \
        _socket

#endif // ANCHOR_H
