/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#ifndef ECGVIEWER_H
#define ECGVIEWER_H

#include <QtDeclarative>
#include <QtGui>

#define POINTS_PER_SECOND 1000

class ECGViewer : public QDeclarativeItem
{
    Q_OBJECT
    Q_DISABLE_COPY(ECGViewer)
    Q_PROPERTY(int duration READ getDuration WRITE setDuration NOTIFY durationChanged)
    Q_PROPERTY(int yInterval READ getYInterval WRITE setYInterval NOTIFY yIntervalChanged)
    Q_PROPERTY(int pWidth READ getPWidth WRITE setPWidth NOTIFY pWidthChanged)
    Q_PROPERTY(QColor color READ getColor WRITE setColor NOTIFY colorChanged)

public:
    explicit ECGViewer();
    /*! Retourne la durée de l'enregistrement. */
    int getDuration() const;
    /*! Retourne l'amplitude de l'ECG en mV. */
    int getYInterval() const;
    /*! Retourne la tailler du painter de la courbe. */
    int getPWidth() const;
    /*! Retourne la couleur de la courbe. */
    QColor getColor() const;
    /*! Définit la durée de l'enregistrement. */
    void setDuration(const int&);
    /*! Définit l'amplitude en mV. */
    void setYInterval(const int&);
    /*! Définit la taille du painter de la courbe. */
    void setPWidth(const int&);
    /*! Définit la couleur de la courbe. */
    void setColor(const QColor&);
    /*! Signal NOTIFY de la durée de l'enregistrement. */
    Q_SIGNAL void durationChanged();
    /*! Signal NOTIFY de l'amplitude en mV. */
    Q_SIGNAL void yIntervalChanged();
    /*! Signal NOTIFY de la taille du painter de la courbe. */
    Q_SIGNAL void pWidthChanged();
    /*! Signal NOTIFY de la couleur de la courbe. */
    Q_SIGNAL void colorChanged();
    Q_INVOKABLE void addPoint(const float&);
    Q_INVOKABLE void refresh();
    Q_INVOKABLE void clear();

protected:
    void paint(QPainter*, const QStyleOptionGraphicsItem*, QWidget*);

private:
    /*! Liste des points de l'ECG. */
    QList<float> _points;
    /*! Durée de l'enregistrement. */
    int _duration;
    /*! Amplitude en mV de l'ECG. */
    int _yInterval;
    /*! Taille du painter de la courbe. */
    int _pWidth;
    /*! Couleur de la courbe. */
    QColor _color;
};

QML_DECLARE_TYPE(ECGViewer)

#endif // ECGVIEWER_H
