/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#include <QtDeclarative>
#include <QDebug>
#include "hooks.h"
#include "anchor.h"

/*!
    \class ConnectionHook
    \brief Le hook ConnectionHook gère l'envoi de données vers l'ancre, située
    entre la partie graphique et logique de l'application.
*/

class ConnectionHook : public GlobalHook
{
public:

    /*!
        \fn ConnectionHook::ConnectionHook()
        \brief Constructeur de la classe.

        Construit un objet ConnectionHook.

        Appelle également GlobalHook() pour enregistrer le hook dans la liste des
        hooks de type \a HOOK_TYPE_GLOBAL.
    */

    ConnectionHook() : GlobalHook() { }

    /*!
        \fn ConnectionHook::onPong(int elapsed)
        \brief Définit la propriété \a latency de l'ancre à \a elapsed.
    */

    void onPong(int elapsed) {
        ANCHOR->setProperty("latency", elapsed);
    }

    /*!
        \fn ConnectionHook::onConnect()
        \brief Définit la propriété \a latency de l'ancre à true.
    */

    void onConnect() {
        ANCHOR->setProperty("connected", true);
    }

    /*!
        \fn ConnectionHook::onDisconnect()
        \brief Définit la propriété \a latency de l'ancre à false.
    */

    void onDisconnect() {
        ANCHOR->setProperty("connected", false);
    }

    /*!
        \fn ConnectionHook::onReconnect(int tries)
        \brief Appelle la fonction reconnectTry() de l'ancre avec le nombre
        d'essais \a tries.
    */

    void onReconnect(int tries) {
        ANCHOR->reconnectTry(tries);
    }

    /*!
        \fn onSocketError(QAbstractSocket::SocketError error)
        \brief Envoit un log contenant l'erreur dans la console.
    */

    void onSocketError(QAbstractSocket::SocketError error) {
        qDebug() << error;
    }
};

/*!
    \fn HookMgr::addGlobalHooks()
    \brief Instancie les hooks de type \a HOOK_TYPE_GLOBAL.
*/

void HookMgr::addGlobalHooks()
{
    new ConnectionHook;
}
