/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#include "hooks.h"
#include "anchor.h"

/*!
    \class HookMgr
    \brief La classe singleton HookMgr est la classe gestionnaire des hooks.

    Afin de garder l'apect généralisé de la classe TcpSocket, la classe HookMgr
    est utilisée pour effectuer toutes les opérations relatives à la fenêtre
    en réponse à la réception d'un message depuis le serveur.

    À noter que cette classe reste généralisée, son rôle étant uniquement
    d'initialiser et de gérer les différents hooks. Elle n'effectue aucun
    traitement direct sur l'interface graphique.
*/

HookMgr *HookMgr::_instance = 0;

/*!
    \fn HookMgr::getInstance()
    \brief Permet d'accéder au pointeur de la classe.
*/

HookMgr &HookMgr::getInstance()
{
    if (!_instance)
        _instance = new HookMgr;
    return *_instance;
}

/*!
    \fn HookMgr::freeInstance()
    \brief Détruit le pointeur de la classe.
*/

void HookMgr::freeInstance()
{
    delete _instance;
    _instance = NULL;
}

/*!
    \fn HookMgr::initialize(Anchor *anchor)
    \brief Initialise le gestionnaire de hooks.

    L'appel de cette fonction permet au gestionnaire de hooks de donner aux
    hooks qu'il gère un accès à l'interface graphique par le biais de \a anchor.
*/

void HookMgr::initialize(Anchor *obj)
{
    anchor = obj;
    addHooks();
}

/*!
    \fn HookMgr::addHooks()
    \brief Ajoute les hooks.

    L'appel de cette fonction permet de créer tous les hooks utilisés par le
    client pour de diverses tâches relatives aux interactions avec le serveur.

    Chaque hook correspond à une instance d'une classe dérivée de QObject. Lors
    de la création d'une telle instance, la fonction registerHook() de HookMgr
    va être appelée afin de l'ajouter aux listes de hooks. Cela permet
    d'ajouter autant de hooks qu'on le souhaite sans pour autant avoir besoin
    de modifier la classe HookMgr.

    Les hooks sont classifiés selon plusieurs types (exemple : GlobalHook) dans
    le sens où un hook assimilé à un module 3D n'aura nullement besoin de se
    préoccuper de l'arrivée d'un pong.
*/

void HookMgr::addHooks()
{
    addGlobalHooks();
    addAccountHooks();
    addModuleHooks();
}

/*!
    \fn HookMgr::registerHook(HookType type, QObject *hook)
    \brief Ajoute le hook à la liste de hooks correspondant à son type.

    L'appel de cette fonction permet au gestionnaire de hooks
    d'appeler par la suite le hook en réponse à une interaction avec le
    serveur.
*/

void HookMgr::registerHook(HookType type, QObject *hook)
{
    switch (type)
    {
        case HOOK_TYPE_GLOBAL:
            if (GlobalHook *h = qobject_cast<GlobalHook*>(hook))
                _globalHooks.append(h);
            break;
        case HOOK_TYPE_ACCOUNT:
            if (AccountHook *h = qobject_cast<AccountHook*>(hook))
                _accountHooks.append(h);
            break;
        case HOOK_TYPE_MODULE:
            if (ModuleHook *h = qobject_cast<ModuleHook*>(hook))
                _moduleHooks.append(h);
            break;
    }
}

/*!
    \fn HookMgr::onPong(int elapsed)
    \brief Appelle la fonction onPong() des GlobalHooks.
*/

void HookMgr::onPong(int elapsed)
{
    foreach (GlobalHook *hook, _globalHooks)
        hook->onPong(elapsed);
}

/*!
    \fn HookMgr::onConnect()
    \brief Appelle la fonction onConnect() des GlobalHooks.
*/

void HookMgr::onConnect()
{
    foreach (GlobalHook *hook, _globalHooks)
        hook->onConnect();
}

/*!
    \fn HookMgr::onDisconnect()
    \brief Appelle la fonction onDisconnect() des GlobalHooks.
*/

void HookMgr::onDisconnect()
{
    foreach (GlobalHook *hook, _globalHooks)
        hook->onDisconnect();
}

/*!
    \fn HookMgr::onReconnect(int tries)
    \brief Appelle la fonction onReconnect(int) des GlobalHooks avec en
    argument le nombre de tentatives effectuées jusqu'alors.
*/

void HookMgr::onReconnect(int tries)
{
    foreach (GlobalHook *hook, _globalHooks)
        hook->onReconnect(tries);
}

/*!
    \fn HookMgr::onSocketError(QAbstractSocket::SocketError error)
    \brief Appelle la fonction onSocketError() des GlobalHooks.
*/

void HookMgr::onSocketError(QAbstractSocket::SocketError error)
{
    foreach (GlobalHook *hook, _globalHooks)
        hook->onSocketError(error);
}

/*!
    \fn HookMgr::onLoginAccepted(bool accepted)
    \brief Appelle la fonction onLoginAccepted() ou onLoginRefused() des
    GlobalHooks, selon le fait que la tentative d'authentification ait aboutie ou non.
*/

void HookMgr::onLoginAccepted(bool accepted)
{
    foreach (AccountHook *hook, _accountHooks) {
        if (accepted)
            hook->onLoginAccepted();
        else
            hook->onLoginRefused();
    }
}

/*!
    \fn HookMgr::onReceiveMedicineModel(const QString &data)
    \brief Appelle la fonction onReceiveMedicineModel() des ModuleHook avec le
    modèle au format JSON contenant la liste des médicaments.
*/

void HookMgr::onReceiveMedicineModel(const QString &data)
{
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveMedicineModel(data);
}

/*!
    \fn HookMgr::onReceivePatientsModel(const QString &data)
    \brief Appelle la fonction onReceivePatientsModel() des ModuleHook avec le
    modèle au format JSON contenant la liste des patients.
*/

void HookMgr::onReceivePatientsModel(const QString &data)
{
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceivePatientsModel(data);
}

/*!
    \fn HookMgr::onReceiveBodyPartsModel(const QString &data)
    \brief Appelle la fonction onReceiveBodyPartsModel() des ModuleHook avec le
    modèle au format JSON contenant les parties du corps.
*/

void HookMgr::onReceiveBodyPartsModel(const QString &data)
{
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveBodyPartsModel(data);
}

/*!
    \fn HookMgr::onReceivePatientInfos(const QString &data)
    \brief Appelle la fonction onReceivePatientInfos() des ModuleHook avec le
    modèle au format JSON contenant les informations du patient.
*/

void HookMgr::onReceivePatientInfos(const QString &data)
{
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceivePatientInfos(data);
}

/*!
    \fn HookMgr::onReceiveNewPatientInfos(const QString &data)
    \brief Appelle la fonction onReceiveNewPatientInfos() des ModuleHook avec le
    modèle au format JSON contenant les informations du patient.
*/

void HookMgr::onReceiveNewPatientInfos(const QString &data)
{
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveNewPatientInfos(data);
}

/*!
    \fn HookMgr::onReceivePatientModifs(int id, const QString &data)
    \brief Appelle la fonction onReceivePatientModifs() des ModuleHook avec le
    modèle au format JSON contenant les informations du patient à l'id \a id.
*/

void HookMgr::onReceivePatientModifs(int id, const QString &data)
{
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceivePatientModifs(id, data);
}

/*!
    \fn HookMgr::onReceiveNewDiagnostic(int id, const QString &data)
    \brief Appelle la fonction onReceiveNewDiagnostic() des ModuleHook avec le
    nouveau diagnostic au format JSON.
*/

void HookMgr::onReceiveNewDiagnostic(int id, const QString &data)
{
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveNewDiagnostic(id, data);
}

/*!
    \fn HookMgr::onReceiveNewSymptom(int id, const QString &data)
    \brief Appelle la fonction onReceiveNewSymptom() des ModuleHook avec le
    nouveau symptôme au format JSON.
*/

void HookMgr::onReceiveNewSymptom(int id, const QString &data)
{
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveNewSymptom(id, data);
}

/*!
    \fn HookMgr::onReceiveRemovePrescription(int patientId, int id)
    \brief Appelle la fonction onReceiveRemovePrescription() des ModuleHook
    avec les informations de la prescription supprimée.
*/

void HookMgr::onReceiveRemovePrescription(int patientId, int id)
{
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveRemovePrescription(patientId, id);
}

/*!
    \fn HookMgr::onReceiveNewPrescription(int id)
    \brief Appelle la fonction onReceiveNewPrescription() des ModuleHook
    avec l'id du patient concerné.
*/

void HookMgr::onReceiveNewPrescription(int id) {
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveNewPrescription(id);
}

/*!
    \fn HookMgr::onReceiveModifyPrescription(int id)
    \brief Appelle la fonction onReceiveModifyPrescription() des ModuleHook
    avec l'id du patient concerné.
*/

void HookMgr::onReceiveModifyPrescription(int id) {
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveModifyPrescription(id);
}

/*!
    \fn HookMgr::onReceiveEnterChat(const QString &username)
    \brief Appelle la fonction onReceiveEnterChat() des ModuleHook.
*/

void HookMgr::onReceiveEnterChat(const QString &username) {
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveEnterChat(username);
}

/*!
    \fn HookMgr::onReceiveLeaveChat(const QString &username)
    \brief Appelle la fonction onReceiveLeaveChat() des ModuleHook.
*/

void HookMgr::onReceiveLeaveChat(const QString &username) {
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveLeaveChat(username);
}

/*!
    \fn HookMgr::onReceiveChatMessage(const QString &username, const QString &message)
    \brief Appelle la fonction onReceiveChatMessage() des ModuleHook.
*/

void HookMgr::onReceiveChatMessage(const QString &username, const QString &message) {
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveChatMessage(username, message);
}

/*!
    \fn HookMgr::onReceiveChatUsers(const QStringList &list)
    \brief Appelle la fonction onReceiveChatUsers() des ModuleHook.
*/

void HookMgr::onReceiveChatUsers(const QStringList &list) {
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveChatUsers(list);
}

/*!
    \fn HookMgr::onReceiveEcgData(const QList<float> &list)
    \brief Appelle la fonction onReceiveEcgData() des ModuleHook avec les
    points de l'ECG.
*/

void HookMgr::onReceiveEcgData(const QList<float> &list) {
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveEcgData(list);
}

/*!
    \fn HookMgr::onReceiveAnalysisModel(const QString &json)
    \brief Appelle la fonction onReceiveAnalysisModel() des ModuleHook avec le
    modèle au format JSON des données des analyses des patients.
*/

void HookMgr::onReceiveAnalysisModel(const QString &json) {
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveAnalysisModel(json);
}

/*!
    \fn HookMgr::onReceiveScan(int id, int part, const QImage &image)
    \brief Appelle la fonction onReceiveScan() des ModuleHook avec l'image de
    la partie du scan.
*/

void HookMgr::onReceiveScan(int id, int part, const QImage &image) {
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveScan(id, part, image);
}

/*!
    \fn HookMgr::onReceiveRadio(int id, int x, int y, const QImage &image)
    \brief Appelle la fonction onReceiveScan() des ModuleHook avec l'image de
    la partie de la radio.
*/

void HookMgr::onReceiveRadio(int id, int x, int y, const QImage &image) {
    foreach (ModuleHook *hook, _moduleHooks)
        hook->onReceiveRadio(id, x, y, image);
}

/*!
    \class GlobalHook
    \brief La classe GlobalHook est la classe de base des hooks dits "globaux".

    Un hook dit "global" est un hook de notifiant des informations à propos de
    l'état de la connexion entre le client et le serveur. On aura par exemple
    la gestion des informations relatives à la latence.
*/

/*!
    \fn GlobalHook::GlobalHook()
    \brief Constructeur de la classe.

    Appelle la fonction HookMgr::registerHook() afin d'ajouter l'instance aux
    listes de hooks.
*/

GlobalHook::GlobalHook()
{
    hookMgr.registerHook(HOOK_TYPE_GLOBAL, this);
}

/*!
    \class AccountHook
    \brief La classe AccountHook est la classe de base des hooks relatifs à la
    gestion d'un compte utilisateur.
*/

/*!
    \fn AccountHook::AccountHook()
    \brief Constructeur de la classe.

    Appelle la fonction HookMgr::registerHook() afin d'ajouter l'instance aux
    listes de hooks.
*/

AccountHook::AccountHook()
{
    hookMgr.registerHook(HOOK_TYPE_ACCOUNT, this);
}

/*!
    \class ModuleHook
    \brief La classe ModuleHook est la classe de base des hooks relatifs aux
    différents modules de l'application.
*/

/*!
    \fn ModuleHook::ModuleHook()
    \brief Constructeur de la classe.

    Appelle la fonction HookMgr::registerHook() afin d'ajouter l'instance aux
    listes de hooks.
*/

ModuleHook::ModuleHook()
{
    hookMgr.registerHook(HOOK_TYPE_MODULE, this);
}
