/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#ifndef HOOKMGR_H
#define HOOKMGR_H

#include <QObject>
#include <QAbstractSocket>
#include <QImage>

class MainWindow;
class Hook;
class GlobalHook;
class AccountHook;
class ModuleHook;
class Anchor;

enum HookType
{
    HOOK_TYPE_GLOBAL,
    HOOK_TYPE_ACCOUNT,
    HOOK_TYPE_MODULE
};

class HookMgr : public QObject
{
    Q_OBJECT

public:
    static HookMgr &getInstance();
    static void freeInstance();
    void initialize(Anchor*);
    void registerHook(HookType, QObject*);
    /*! Ancre entre le C++ et l'interface graphique. */
    Anchor *anchor;

public slots:
    void onPong(int);
    void onConnect();
    void onReconnect(int);
    void onDisconnect();
    void onSocketError(QAbstractSocket::SocketError);
    void onLoginAccepted(bool);
    void onReceiveMedicineModel(const QString&);
    void onReceivePatientsModel(const QString&);
    void onReceiveBodyPartsModel(const QString&);
    void onReceivePatientInfos(const QString&);
    void onReceiveNewPatientInfos(const QString&);
    void onReceivePatientModifs(int, const QString&);
    void onReceiveNewDiagnostic(int, const QString&);
    void onReceiveNewSymptom(int, const QString&);
    void onReceiveRemovePrescription(int, int);
    void onReceiveNewPrescription(int);
    void onReceiveModifyPrescription(int);
    void onReceiveEnterChat(const QString&);
    void onReceiveLeaveChat(const QString&);
    void onReceiveChatMessage(const QString&, const QString&);
    void onReceiveChatUsers(const QStringList&);
    void onReceiveEcgData(const QList<float>&);
    void onReceiveAnalysisModel(const QString&);
    void onReceiveScan(int, int, const QImage&);
    void onReceiveRadio(int, int, int, const QImage&);

private:
    /*! Constructeur. */
    explicit HookMgr() { }
    /*! Destructeur. */
    ~HookMgr() { }
    /*! Pointeur unique. */
    static HookMgr *_instance;

    void addHooks();
    void addGlobalHooks();
    void addAccountHooks();
    void addModuleHooks();

    /*! Liste des hooks globaux. */
    QList<GlobalHook*> _globalHooks;
    /*! Liste des hooks du compte utilisateur. */
    QList<AccountHook*> _accountHooks;
    /*! Liste des hooks des modules. */
    QList<ModuleHook*> _moduleHooks;
};

/*! Macro vers HookMgr::getInstance(). */
#define hookMgr HookMgr::getInstance()
/*! Macro d'accès à l'ancre entre le C++ et QML */
#define ANCHOR \
    if (hookMgr.anchor) \
        hookMgr.anchor

class GlobalHook : public QObject
{
    Q_OBJECT

protected:
    GlobalHook();

public:
    /*! Appelée lors de la réception d'un pong. */
    virtual void onPong(int) { }
    /*! Appelée lors de la réussite d'une connexion. */
    virtual void onConnect() { }
    /*! Appelée lors d'une tentative de reconnexion. */
    virtual void onReconnect(int) { }
    /*! Appelée lors d'une déconnexion. */
    virtual void onDisconnect() { }
    /*! Appelée lors d'une erreur de socket. */
    virtual void onSocketError(QAbstractSocket::SocketError) { }
};

class AccountHook : public QObject
{
    Q_OBJECT

protected:
    AccountHook();

public:
    /*! Appelée lors de la réussite d'une tentative d'authentification. */
    virtual void onLoginAccepted() { }
    /*! Appelée lors de l'échec d'une tentative d'authentification. */
    virtual void onLoginRefused() { }
};

class ModuleHook : public QObject
{
    Q_OBJECT

protected:
    ModuleHook();

public:
    /*! Appelée lors de la réception du modèle contenant les médicaments. */
    virtual void onReceiveMedicineModel(const QString&) { }
    /*! Appelée lors de la réception du modèle contenant les patients. */
    virtual void onReceivePatientsModel(const QString&) { }
    /*! Appelée lors de la réception du modèle des parties du corps. */
    virtual void onReceiveBodyPartsModel(const QString&) { }
    /*! Appelée lors de la réception des informations d'un patient. */
    virtual void onReceivePatientInfos(const QString&) { }
    /*! Appelée lors d'une notification d'ajout du dossier d'un patient. */
    virtual void onReceiveNewPatientInfos(const QString&) { }
    /*! Appelée lors d'une modification du dossier d'un patient. */
    virtual void onReceivePatientModifs(int, const QString&) { }
    /*! Appelée lors de l'ajout d'un diagnostic au dossier d'un patient. */
    virtual void onReceiveNewDiagnostic(int, const QString&) { }
    /*! Appelée lors de l'ajout d'un symptôme au dossier d'un patient. */
    virtual void onReceiveNewSymptom(int, const QString&) { }
    /*! Appelée lors du retrait d'une prescription du dossier d'un patient. */
    virtual void onReceiveRemovePrescription(int, int) { }
    /*! Appelée lors de l'ajout d'une prescription du dossier d'un patient. */
    virtual void onReceiveNewPrescription(int) { }
    /*! Appelée lors de la modification d'une prescription. */
    virtual void onReceiveModifyPrescription(int) { }
    /*! Appelée lors de l'entrée d'un utilisateur dans le salon. */
    virtual void onReceiveEnterChat(const QString&) { }
    /*! Appelée lors de la sortie d'un utilisateur du salon. */
    virtual void onReceiveLeaveChat(const QString&) { }
    /*! Appelée lors de la réception d'un nouveau message dans le salon. */
    virtual void onReceiveChatMessage(const QString&, const QString&) { }
    /*! Appelée lors de la réception d'un nouveau message dans le salon. */
    virtual void onReceiveChatUsers(const QStringList&) { }
    /*! Appelée lors de la réception de points de l'ECG. */
    virtual void onReceiveEcgData(const QList<float>&) { }
    /*! Appelée lors de la réception du modèle des résultats des analyses. */
    virtual void onReceiveAnalysisModel(const QString&) { }
    /*! Appelée lors de la réception de l'image d'un scan. */
    virtual void onReceiveScan(int, int, const QImage&) { }
    /*! Appelée lors de la réception de l'image d'une radio. */
    virtual void onReceiveRadio(int, int, int, const QImage&) { }
};

#endif // HOOKMGR_H
