/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#include <QtDeclarative>
#include "hooks.h"
#include "anchor.h"

/*!
    \class PatientFollowUpHook
    \brief Le hook PatientFollowUpHook gère l'envoi de données vers l'ancre,
    située entre la partie graphique et logique de l'application.
*/

class PatientFollowUpHook : public ModuleHook
{
public:

    /*!
        \fn PatientFollowUpHook::PatientFollowUpHook()
        \brief Constructeur de la classe.

        Construit un objet PatientFollowUpHook.

        Appelle également ModuleHook() pour enregistrer le hook dans la liste des
        hooks de type \a HOOK_TYPE_GLOBAL.
    */

    PatientFollowUpHook() : ModuleHook() { }

    /*!
        \fn PatientFollowUpHook::onReceiveMedicineModel(const QString &data)
        \brief Appelle le signal receiveMedicineModel() de l'ancre avec le
        modèle des médicaments au format JSON.
    */

    void onReceiveMedicineModel(const QString &data) {
        ANCHOR->receiveMedicineModel(data);
    }

    /*!
        \fn PatientFollowUpHook::onReceivePatientsModel(const QString &data)
        \brief Appelle le signal receiveMedicineModel() de l'ancre avec le
        modèle des patients au format JSON.
    */

    void onReceivePatientsModel(const QString &data) {
        ANCHOR->receivePatientsModel(data);
    }

    /*!
        \fn PatientFollowUpHook::onReceivePatientInfos(const QString &data)
        \brief Appelle le signal receiveMedicineModel() de l'ancre avec les
        informations d'un patient au format JSON.
    */

    void onReceivePatientInfos(const QString &data) {
        ANCHOR->receivePatientInfos(data);
    }

    /*!
        \fn PatientFollowUpHook::onReceiveNewPatientInfos(const QString &data)
        \brief Appelle le signal receiveNewPatientInfos() de l'ancre avec
        les informations du nouveau patient au format JSON.
    */

    void onReceiveNewPatientInfos(const QString &data) {
        ANCHOR->receiveNewPatientInfos(data);
    }

    /*!
        \fn PatientFollowUpHook::onReceivePatientModifs(int id, const QString &data)
        \brief Appelle le signal receivePatientModifs() de l'ancre avec les
        informations du patient à l'id \a id au format JSON.
    */

    void onReceivePatientModifs(int id, const QString &data) {
        ANCHOR->receivePatientModifs(id, data);
    }

    /*!
        \fn PatientFollowUpHook::onReceiveNewDiagnostic(int id, const QString &data)
        \brief Appelle le signal receiveNewDiagnostic() de l'ancre avec les
        informations du diagnostic au format JSON.
    */

    void onReceiveNewDiagnostic(int id, const QString &data) {
        ANCHOR->receiveNewDiagnostic(id, data);
    }

    /*!
        \fn PatientFollowUpHook::onReceiveNewSymptom(int id, const QString &data)
        \brief Appelle le signal receiveNewSymptom() de l'ancre avec les
        informations du symptôme au format JSON.
    */

    void onReceiveNewSymptom(int id, const QString &data) {
        ANCHOR->receiveNewSymptom(id, data);
    }

    /*!
        \fn PatientFollowUpHook::onReceiveRemovePrescription(int patientId, int id)
        \brief Appelle le signal receiveRemovePrescription() de l'ancre avec les
        informations de la prescription supprimée.
    */

    void onReceiveRemovePrescription(int patientId, int id) {
        ANCHOR->receiveRemovePrescription(patientId, id);
    }

    /*!
        \fn PatientFollowUpHook::onReceiveNewPrescription(int id)
        \brief Appelle le signal receiveNewPrescription() de l'ancre avec l'id
        du patient concerné.
    */

    void onReceiveNewPrescription(int id) {
        ANCHOR->receiveNewPrescription(id);
    }

    /*!
        \fn PatientFollowUpHook::onReceiveModifyPrescription(int id)
        \brief Appelle le signal receiveModifyPrescription() de l'ancre avec
        l'id du patient concerné.
    */

    void onReceiveModifyPrescription(int id) {
        ANCHOR->receiveModifyPrescription(id);
    }
};

/*!
    \class ChatHook
    \brief Le hook ChatHook gère l'envoi de données relatives au chat vers
    l'ancre, située entre la partie graphique et logique de l'application.
*/

class ChatHook : public ModuleHook
{
public:

    /*!
        \fn ChatHook::ChatHook()
        \brief Constructeur de la classe.

        Construit un objet ChatHook.

        Appelle également ModuleHook() pour enregistrer le hook dans la liste des
        hooks de type \a HOOK_TYPE_GLOBAL.
    */

    ChatHook() : ModuleHook() { }

    /*!
        \fn ChatHook::onReceiveEnterChat(const QString &username)
        \brief Appelle le signal receiveEnterChat() de l'ancre avec le nom de
        l'utilisateur.
    */

    void onReceiveEnterChat(const QString &username) {
        ANCHOR->receiveEnterChat(username);
    }

    /*!
        \fn ChatHook::onReceiveLeaveChat(const QString &username)
        \brief Appelle le signal receiveLeaveChat() de l'ancre avec le nom de
        l'utilisateur.
    */

    void onReceiveLeaveChat(const QString &username) {
        ANCHOR->receiveLeaveChat(username);
    }

    /*!
        \fn ChatHook::onReceiveChatMessage(const QString &username, const QString &message)
        \brief Appelle le signal receiveChatMessage() de l'ancre avec le nom de
        l'utilisateur et le message.
    */

    void onReceiveChatMessage(const QString &username, const QString &message) {
        ANCHOR->receiveChatMessage(username, message);
    }

    /*!
        \fn ChatHook::onReceiveChatMessage(const QString &username, const QString &message)
        \brief Appelle le signal receiveChatMessage() de l'ancre avec le nom de
        l'utilisateur et le message.
    */

    void onReceiveChatUsers(const QStringList &list) {
        ANCHOR->receiveChatUsers(list);
    }

    void onReceiveEcgData(const QList<float> &list) {
        QStringList data;
        foreach(float value, list)
            data.append(QString::number(value));
        ANCHOR->receiveEcgData(data);
    }
};

/*!
    \class AnalysisHook
    \brief Le hook AnalysisHook gère l'envoi de données vers l'ancre,
    située entre la partie graphique et logique de l'application.
*/

class AnalysisHook : public ModuleHook
{
public:

    /*!
        \fn AnalysisHook::AnalysisHook()
        \brief Constructeur de la classe.

        Construit un objet AnalysisHook.

        Appelle également ModuleHook() pour enregistrer le hook dans la liste des
        hooks de type \a HOOK_TYPE_GLOBAL.
    */

    AnalysisHook() : ModuleHook() { }

    void onReceiveBodyPartsModel(const QString &data) {
        ANCHOR->receiveBodyPartsModel(data);
    }

    /*!
        \fn AnalysisHook::onReceiveAnalysisModel(const QString &data)
        \brief Appelle le signal receiveAnalysisModel() de l'ancre avec le
        modèle des analyses au format JSON.
    */

    void onReceiveAnalysisModel(const QString &data) {
        ANCHOR->receiveAnalysisModel(data);
    }

    /*!
        \fn AnalysisHook::onReceiveScan(int id, int part, const QImage &scan)
        \brief Sauvegarde l'image dans le dossier contenant les scans puis
        appelle le signal receiveScan() de l'ancre avec le chemin vers
        l'image ainsi sauvegardée.
    */

    void onReceiveScan(int id, int part, const QImage &scan) {
        QDir dir(qApp->applicationDirPath());
        if (!dir.exists("scans"))
            dir.mkdir("scans");
        if (!scan.isNull()) {
            QString path = qApp->applicationDirPath() + "/scans/scan_"
                + QString::number(id) + "_" + QString::number(part) + ".jpg";
            scan.save(path);
            ANCHOR->receiveScan(id, part, path);
            return;
        }
        ANCHOR->receiveScan(id, part, "");
    }

    void onReceiveRadio(int id, int x, int y, const QImage &radio) {
        QDir dir(qApp->applicationDirPath());
        if (!dir.exists("radios"))
            dir.mkdir("radios");
        if (!radio.isNull()) {
            QString path = qApp->applicationDirPath() + "/radios/radio_"
                + QString::number(id) + "_" + QString::number(x) + "_"
                + QString::number(y) + ".jpg";
            radio.save(path);
            ANCHOR->receiveRadio(id, x, y, path);
            return;
        }
        ANCHOR->receiveRadio(id, x, y, "");
    }
};

/*!
    \fn HookMgr::addModuleHooks()
    \brief Instancie les hooks de type \a HOOK_TYPE_MODULE.
*/

void HookMgr::addModuleHooks()
{
    new PatientFollowUpHook;
    new ChatHook;
    new AnalysisHook;
}

