/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#include "tcpsocket.h"
#include "opcodes.h"
#include "hooks.h"

/*!
    \class TcpSocket
    \brief La classe TcpSocket est la classe correspondant à un client.

    Dérivée de QTcpSocket, elle se connecte sur le port 45354, IP locale, et
    envoit des paquets, identifiables par leur opcode, au serveur selon les
    circonstances.

    Après être connectée au serveur, l'application va envoyer toutes les dix
    secondes un ping au serveur pour informer de la latence.
*/

/*! Intervalle entre deux pings. */
const int pingInterval = 10000;

/*!
    \fn TcpSocket::TcpSocket()
    \brief Constructeur de la classe.

    Construit un objet TcpSocket.
*/

TcpSocket::TcpSocket() : _size(0), _reconnectTries(1)
{
    connectToHost("127.0.0.1", 45354);

    connect(&_pingTimer, SIGNAL(timeout()), this, SLOT(sendPing()));
    connect(this, SIGNAL(readyRead()), this, SLOT(read()));
    connect(this, SIGNAL(disconnected()), &hookMgr, SLOT(onDisconnect()));
    connect(this, SIGNAL(connected()), this, SLOT(handleConnected()));
    connect(this, SIGNAL(error(QAbstractSocket::SocketError)), &hookMgr,
        SLOT(onSocketError(QAbstractSocket::SocketError)));

    _pingTimer.setInterval(pingInterval);
    _pingTimer.start();
}

/*!
    \fn TcpSocket::handleConnected()
    \brief Envoit au gestionnaire de hooks une notification de connexion.
*/

void TcpSocket::handleConnected()
{
    _reconnectTries = 1;
    hookMgr.onConnect();
}

/*!
    \fn TcpSocket::reconnect()
    \brief Reconnecte le client au serveur.
*/

void TcpSocket::reconnect()
{
    _reconnectTries++;
    hookMgr.onReconnect(_reconnectTries);
    abort();
    connectToHost("127.0.0.1", 45354);
    _pingTimer.start();
}

/*!
    \fn TcpSocket::read()
    \brief Fonction de lecture des paquets.

    Récupère progressivement le contenu du paquet envoyé par le serveur, puis
    appelle la fonction privée handleMessage() pour que celle-ci le gère. Cela
    fait, la fonction nettoie les données restantes pour préparer l'arrivée
    d'un nouveau paquet.

    \sa TcpSocket::handleMessage(QDataStream *data)
*/

void TcpSocket::read()
{
    QDataStream in(this);
    quint64 _bytesAvailable = bytesAvailable();

    if (_size == 0) {
        if (_bytesAvailable < (int)sizeof(quint64))
            return;
        in >> _size;
    }

    if (_bytesAvailable < _size)
        return;

    handleMessage(&in);
    readAll();
    _size = 0;
}

/*!
    \fn TcpSocket::handleMessage(QDataStream *data)
    \brief Gestionnaire global des paquets.

    Cette fonction est appelée une fois qu'un paquet a été entièrement reçu
    afin de le rediriger vers le gestionnaire de paquets associé à l'opcode
    qu'il possède.

    Dans un premier temps, on récupère l'opcode, puis on vérifie qu'elle est
    bien valide. Enfin, on appelle le gestionnaire associé.
*/

void TcpSocket::handleMessage(QDataStream *data)
{
    int opcode;
    *data >> opcode;

    if (opcode >= MSG_MAX_OPCODES)
        return;

    OpcodeHandler& handle = opcodes[opcode];
    (this->*handle.handler)(data);
}

/*!
    \fn TcpSocket::unhandled(QDataStream *data)
    \brief Gestionnaire des paquets inutilisés.

    Ne fait rien.
*/

void TcpSocket::unhandled(QDataStream *data)
{
    Q_UNUSED(data);
}

/*!
    \fn TcpSocket::handlePong(QDataStream *data)
    \brief Gestionnaire des paquets correspondant aux pongs.

    Récupère un pong envoyé par le serveur suite à un ping.
*/

void TcpSocket::handlePong(QDataStream *data)
{
    Q_UNUSED(data);
    hookMgr.onPong(_pongTime.elapsed());
}

/*!
    \fn TcpSocket::handleLogin(QDataStream *data)
    \brief Gestionnaire des paquets correspondant au résultat d'une tentative
    de login.

    Retourne true en cas de succès, sinon false.
*/

void TcpSocket::handleLogin(QDataStream *data)
{
    bool answer;
    *data >> answer;
    hookMgr.onLoginAccepted(answer);
}

/*!
    \fn TcpSocket::handleMedicineModel(QDataStream *data)
    \brief Gestionnaire des paquets correspondant aux retour d'une demande de
    modèle contenant les médicaments disponibles.

    Les données retournées sont au format JSON et sont parsées par la suite
    dans le QML.
*/

void TcpSocket::handleMedicineModel(QDataStream *data)
{
    QString medicine;
    *data >> medicine;
    hookMgr.onReceiveMedicineModel(medicine);
}

/*!
    \fn TcpSocket::handlePatientsModel(QDataStream *data)
    \brief Gestionnaire des paquets correspondant aux retour d'une demande de
    modèle contenant les patients.

    Les données retournées sont au format JSON et sont parsées par la suite
    dans le QML.
*/

void TcpSocket::handlePatientsModel(QDataStream *data)
{
    QString patients;
    *data >> patients;
    hookMgr.onReceivePatientsModel(patients);
}

/*!
    \fn TcpSocket::handleBodyPartsModel(QDataStream *data)
    \brief Gestionnaire des paquets correspondant aux retour d'une demande de
    modèle contenant les parties du corps.

    Les données retournées sont au format JSON et sont parsées par la suite
    dans le QML.
*/

void TcpSocket::handleBodyPartsModel(QDataStream *data)
{
    QString parts;
    *data >> parts;
    hookMgr.onReceiveBodyPartsModel(parts);
}

/*!
    \fn TcpSocket::handlePatientQuery(QDataStream *data)
    \brief Gestionnaire des paquets correspondant aux retour d'une demande
    d'informations à propos d'un patient.

    Les données retournées sont au format JSON et sont parsées par la suite
    dans le QML.
*/

void TcpSocket::handlePatientQuery(QDataStream *data)
{
    QString patientData;
    *data >> patientData;
    hookMgr.onReceivePatientInfos(patientData);
}

/*!
    \fn TcpSocket::handleNewPatient(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification d'ajout
    du dossier d'un patient.

    Les données retournées sont au format JSON et sont parsées par la suite
    dans le QML.
*/

void TcpSocket::handleNewPatient(QDataStream *data) {
    QString patientData;
    *data >> patientData;
    hookMgr.onReceiveNewPatientInfos(patientData);
}

/*!
    \fn TcpSocket::handleModifyPatient(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification de
    modification du dossier d'un patient.

    Les données retournées sont au format JSON et sont parsées par la suite
    dans le QML.
*/

void TcpSocket::handleModifyPatient(QDataStream *data) {
    int id;
    QString patientData;
    *data >> id >> patientData;
    hookMgr.onReceivePatientModifs(id, patientData);
}

/*!
    \fn TcpSocket::handleNewDiagnostic(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification d'ajout
    de diagnostic dans le dossier d'un patient.

    Les données retournées sont au format JSON et sont parsées par la suite
    dans le QML.
*/

void TcpSocket::handleNewDiagnostic(QDataStream *data)
{
    int id;
    QString diagnostic;
    *data >> id >> diagnostic;
    hookMgr.onReceiveNewDiagnostic(id, diagnostic);
}

/*!
    \fn TcpSocket::handleNewSymptom(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification d'ajout
    de symptôme dans le dossier d'un patient.

    Les données retournées sont au format JSON et sont parsées par la suite
    dans le QML.
*/

void TcpSocket::handleNewSymptom(QDataStream *data)
{
    int id;
    QString symptom;
    *data >> id >> symptom;
    hookMgr.onReceiveNewSymptom(id, symptom);
}

/*!
    \fn TcpSocket::handleRemovePrescription(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification de
    suppression de prescription.

    Les données retournées sont au format JSON et sont parsées par la suite
    dans le QML.
*/

void TcpSocket::handleRemovePrescription(QDataStream *data)
{
    int patientId, id;
    *data >> patientId >> id;
    hookMgr.onReceiveRemovePrescription(patientId, id);
}

/*!
    \fn TcpSocket::handleNewPrescription(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification d'ajout de
    prescription.
*/

void TcpSocket::handleNewPrescription(QDataStream *data)
{
    int id;
    *data >> id;
    hookMgr.onReceiveNewPrescription(id);
}

/*!
    \fn TcpSocket::handleModifyPrescription(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification de
    modification de prescription.
*/

void TcpSocket::handleModifyPrescription(QDataStream *data)
{
    int id;
    *data >> id;
    hookMgr.onReceiveModifyPrescription(id);
}

/*!
    \fn TcpSocket::handleEnterChat(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification d'entrée
    d'un client dans le salon.
*/

void TcpSocket::handleEnterChat(QDataStream *data)
{
    QString username;
    *data >> username;
    hookMgr.onReceiveEnterChat(username);
}

/*!
    \fn TcpSocket::handleLeaveChat(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification de sortie
    d'un client du salon.
*/

void TcpSocket::handleLeaveChat(QDataStream *data)
{
    QString username;
    *data >> username;
    hookMgr.onReceiveLeaveChat(username);
}

/*!
    \fn TcpSocket::handleChatMessage(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification d'ajout de
    message dans le chat.
*/

void TcpSocket::handleChatMessage(QDataStream *data)
{
    QString username, message;
    *data >> username >> message;
    hookMgr.onReceiveChatMessage(username, message);
}

/*!
    \fn TcpSocket::handleChatUsersRequest(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification de
    réception de la liste des utilisateurs présents dans le chat.
*/

void TcpSocket::handleChatUsersRequest(QDataStream *data)
{
    QStringList list;
    *data >> list;
    hookMgr.onReceiveChatUsers(list);
}

/*!
    \fn TcpSocket::handleEcgData(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification de
    réception de points pour la courbe de l'ECG.
*/

void TcpSocket::handleEcgData(QDataStream *data)
{
    QList<float> list;
    *data >> list;
    hookMgr.onReceiveEcgData(list);
}

/*!
    \fn TcpSocket::handleAnalysisModel(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification de
    réception du modèle JSON des analyses (radios et scans) des patients.
*/

void TcpSocket::handleAnalysisModel(QDataStream *data)
{
    QString json;
    *data >> json;
    hookMgr.onReceiveAnalysisModel(json);
}

/*!
    \fn TcpSocket::handleScanRequest(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification de
    réception de l'image d'une partie de scan.
*/

void TcpSocket::handleScanRequest(QDataStream *data)
{
    int id, part;
    QImage image;
    *data >> id >> part >> image;
    hookMgr.onReceiveScan(id, part, image);
}

/*!
    \fn TcpSocket::handleRadioRequest(QDataStream *data)
    \brief Gestionnaire des paquets correspondant à une notification de
    réception de l'image d'une partie de radio.
*/

void TcpSocket::handleRadioRequest(QDataStream *data)
{
    int id, x, y;
    QImage image;
    *data >> id >> x >> y >> image;
    hookMgr.onReceiveRadio(id, x, y, image);
}

/*!
    \fn TcpSocket::sendEmptyMessage(int opcode)
    \brief Envoit un message simple au serveur contenant une opcode et les
    données obligatoires d'un paquet en provenance d'un client.
*/

void TcpSocket::sendEmptyMessage(int opcode)
{
    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint16)0;
    out << opcode;
    out << QString(CLIENT_USERNAME);
    out << QString(CLIENT_PASSWORD);
    out.device()->seek(0);
    out << (quint16)(packet.size() - sizeof(quint16));

    write(packet);
}

/*!
    \fn TcpSocket::sendPing()
    \brief Envoit un ping au serveur et gère les tentatives de reconnexion.
*/

void TcpSocket::sendPing()
{
    if (state() == QAbstractSocket::UnconnectedState) {
        reconnect();
        return;
    }
    sendEmptyMessage(MSG_PING);
    _pongTime.restart();
}

/*!
    \fn TcpSocket::sendLogin(QString username, QString password)
    \brief Envoit les éléments d'une tentative de connexion au serveur.
*/

void TcpSocket::sendLogin(QString username, QString password)
{
    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint16)0;
    out << int(MSG_LOGIN);
    out << QString(CLIENT_USERNAME);
    out << QString(CLIENT_PASSWORD);
    out << username;
    out << password;
    out.device()->seek(0);
    out << (quint16)(packet.size() - sizeof(quint16));

    write(packet);
}

/*!
    \fn TcpSocket::sendRequestForMedicineModel()
    \brief Envoit une demande du modèle des médicaments au serveur.
*/

void TcpSocket::sendRequestForMedicineModel()
{
    sendEmptyMessage(MSG_QUERY_MEDICINE_MODEL);
}

/*!
    \fn TcpSocket::sendRequestForPatients()
    \brief Envoit une demande du modèle de tous les patients au serveur.
*/

void TcpSocket::sendRequestForPatients()
{
    sendEmptyMessage(MSG_QUERY_PATIENTS_MODEL);
}

/*!
    \fn TcpSocket::sendRequestForBodyParts()
    \brief Envoit une demande du modèle des parties du corps humain.
*/

void TcpSocket::sendRequestForBodyParts()
{
    sendEmptyMessage(MSG_QUERY_BODY_PARTS);
}

/*!
    \fn TcpSocket::sendRequestForPatient(int id)
    \brief Envoit une demande des informations du patient \a id.
*/

void TcpSocket::sendRequestForPatient(int id)
{
    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint16)0;
    out << int(MSG_QUERY_PATIENT);
    out << QString(CLIENT_USERNAME);
    out << QString(CLIENT_PASSWORD);
    out << id;
    out.device()->seek(0);
    out << (quint16)(packet.size() - sizeof(quint16));

    write(packet);
}

/*!
    \fn TcpSocket::sendPatientInfos(int id, QStringList data)
    \brief Envoit les informations relative à un ajout ou une modification du
    dossier d'un patient.
*/

void TcpSocket::sendPatientInfos(int id, QStringList data)
{
    int opcode = id == -1 ? MSG_NEW_PATIENT : MSG_MODIFY_PATIENT;

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint16)0;
    out << opcode;
    out << QString(CLIENT_USERNAME);
    out << QString(CLIENT_PASSWORD);
    if (id != -1)
        out << id;
    out << data;
    out.device()->seek(0);
    out << (quint16)(packet.size() - sizeof(quint16));

    write(packet);
}

/*!
    \fn TcpSocket::sendNewDiagnostic(int id, QString data)
    \brief Envoit les données relatives à l'ajout d'un diagnostic au dossier
    du patient à l'id \a id.
*/

void TcpSocket::sendNewDiagnostic(int id, QString data)
{
    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint16)0;
    out << int(MSG_NEW_DIAGNOSTIC);
    out << QString(CLIENT_USERNAME);
    out << QString(CLIENT_PASSWORD);
    out << id;
    out << data;
    out.device()->seek(0);
    out << (quint16)(packet.size() - sizeof(quint16));

    write(packet);
}

/*!
    \fn TcpSocket::sendNewSymptom(int id, QString data)
    \brief Envoit les données relatives à l'ajout d'un symptôme au dossier
    du patient à l'id \a id.
*/

void TcpSocket::sendNewSymptom(int id, QString data)
{
    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint16)0;
    out << int(MSG_NEW_SYMPTOM);
    out << QString(CLIENT_USERNAME);
    out << QString(CLIENT_PASSWORD);
    out << id;
    out << data;
    out.device()->seek(0);
    out << (quint16)(packet.size() - sizeof(quint16));

    write(packet);
}

/*!
    \fn TcpSocket::sendRemovePrescription(int patientId, int id)
    \brief Envoit les données relatives à la suppression de la prescription
    \a id du dossier du patient à l'id \a patientId.
*/

void TcpSocket::sendRemovePrescription(int patientId, int id)
{
    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint16)0;
    out << int(MSG_REMOVE_PRESCRIPTION);
    out << QString(CLIENT_USERNAME);
    out << QString(CLIENT_PASSWORD);
    out << patientId;
    out << id;
    out.device()->seek(0);
    out << (quint16)(packet.size() - sizeof(quint16));

    write(packet);
}

/*!
    \fn TcpSocket::sendPrescription(int patientId, int id, QString duration, int date, int medicineId)
    \brief Envoit les données relatives à la modification ou à l'ajout d'une
    prescription du dossier du patient à l'id \a patientId.
*/

void TcpSocket::sendPrescription(int patientId, int id, QString duration, int date, int medicineId)
{
    int opcode = id == -1 ? MSG_NEW_PRESCRIPTION : MSG_MODIFY_PRESCRIPTION;

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint16)0;
    out << opcode;
    out << QString(CLIENT_USERNAME);
    out << QString(CLIENT_PASSWORD);
    out << patientId;
    if (id != -1)
        out << id;
    out << duration;
    out << date;
    out << medicineId;
    out.device()->seek(0);
    out << (quint16)(packet.size() - sizeof(quint16));

    write(packet);
}

/*!
    \fn TcpSocket::sendEnterChat()
    \brief Envoit les données relatives à l'entrée de l'utilisateur du client
    dans le salon.
*/

void TcpSocket::sendEnterChat()
{
    sendEmptyMessage(MSG_ENTER_CHAT);
}

/*!
    \fn TcpSocket::sendLeaveChat()
    \brief Envoit les données relatives à la sortie de l'utilisateur du client
    du salon.
*/

void TcpSocket::sendLeaveChat()
{
    sendEmptyMessage(MSG_LEAVE_CHAT);
}

/*!
    \fn TcpSocket::sendMessageInChat(QString message)
    \brief Envoit un message dans le salon.
*/

void TcpSocket::sendMessageInChat(QString message)
{
    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint16)0;
    out << int(MSG_SEND_CHAT_MESSAGE);
    out << QString(CLIENT_USERNAME);
    out << QString(CLIENT_PASSWORD);
    out << message;
    out.device()->seek(0);
    out << (quint16)(packet.size() - sizeof(quint16));

    write(packet);
}

/*!
    \fn TcpSocket::sendChatUsersRequest()
    \brief Envoit une demande de la liste des utilisateurs connectés au chat.
*/

void TcpSocket::sendChatUsersRequest()
{
    sendEmptyMessage(MSG_QUERY_CHAT_USERS);
}

/*!
    \fn TcpSocket::sendStartECG(int index)
    \brief Envoit au serveur une demande de lancement de l'ECG.
*/

void TcpSocket::sendStartECG(int index)
{
    Q_UNUSED(index);
    sendEmptyMessage(MSG_START_ECG);
}

/*!
    \fn TcpSocket::sendStopECG()
    \brief Envoit au serveur une demande d'arrêt de l'ECG.
*/

void TcpSocket::sendStopECG()
{
    sendEmptyMessage(MSG_STOP_ECG);
}

/*!
    \fn TcpSocket::sendRequestForAnalysis()
    \brief Envoit au serveur une demande du modèle des analyses (radios et
    scans) des patients.
*/

void TcpSocket::sendRequestForAnalysis()
{
    sendEmptyMessage(MSG_QUERY_ANALYSIS_MODEL);
}

/*!
    \fn TcpSocket::sendRequestForScan(int id, int part)
    \brief Envoit une demande de la partie \a part du scan à l'id \a id.
*/

void TcpSocket::sendRequestForScan(int id, int part)
{
    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint16)0;
    out << int(MSG_QUERY_SCAN);
    out << QString(CLIENT_USERNAME);
    out << QString(CLIENT_PASSWORD);
    out << id;
    out << part;
    out.device()->seek(0);
    out << (quint16)(packet.size() - sizeof(quint16));

    write(packet);
}

/*!
    \fn TcpSocket::sendRequestForRadio(int id, int x, int y)
    \brief Envoit une demande de la partie (\a x, \a y) de la radio à
    l'id \a id.
*/

void TcpSocket::sendRequestForRadio(int id, int x, int y)
{
    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint16)0;
    out << int(MSG_QUERY_RADIO);
    out << QString(CLIENT_USERNAME);
    out << QString(CLIENT_PASSWORD);
    out << id;
    out << x;
    out << y;
    out.device()->seek(0);
    out << (quint16)(packet.size() - sizeof(quint16));

    write(packet);
}
