/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

import QtQuick 1.0
import Utopital 1.0
import "Components"
import "Delegates"
import "Popups"

Rectangle {
    id: module
    anchors.fill: parent
    color: "#545454"
    state: "choiceOfType"

    property int patient: -1
    property int patientCurrentlyViewed: -1
    property bool firstTime: true
    property bool popupWaitVisible: false
    property bool fullscreenScansVisible: false
    property bool popupRadioInfosVisible: false
    property string type: ""
    property int scanCurrentlyViewed: 0

    function patientChoosed(index) {
        var model = module.type == "scans" ? scansModel : radiosModel;
        patient = model.get(index).customer_id;
        patientCurrentlyViewed = index;
        if (type == "scans") {
            popupWaitVisible = true;
            state = "viewScans";
            main.getScans(patientCurrentlyViewed);
        }
        else {
            popupWaitVisible = true;
            state = "viewRadios";
            main.getRadios(patientCurrentlyViewed);
        }
    }

    function scanModelFilled() {
        popupWaitVisible = false;
    }

    function radioModelFilled() {
        popupWaitVisible = false;
    }

    onVisibleChanged: {
        if (visible && firstTime) {
            anchor.requestForAnalysisModel();
            firstTime = false;
        }
        type = "";
        state = "choiceOfType";
    }

    WaitPopup {
        visible: popupWaitVisible
        opacity: visible ? 1 : 0
        text: "Récupération des données depuis le serveur..."
            + "\nVeuillez patienter."
        z: 5
    }

    Background { }

    MenuAccessor { }

    Menus {
        id: menusArea
        contentHeight: contentChoice.height
    }

    PopupInformation {
        title: "Détails de la radio"
        visible: module.popupRadioInfosVisible
        opacity: visible ? 1 : 0
        onHideRequest: module.popupRadioInfosVisible = false

        onVisibleChanged: {
            var tmp = "Radio de " + (patientsModel.count > module.patient
                && patientsModel.get(module.patient)
                ? patientsModel.get(module.patient).name
                : "") + "<br />Partie : ";
            var radio = radiosModel.get(module.patientCurrentlyViewed);
            if (visible && radio) {
                var bodyPart = bodyPartsModel.get(radio.group_id);
                if (bodyPart) {
                    text = tmp + bodyPart.name;
                    return;
                }
            }
            text = tmp + "indéterminée";
        }
    }

    Item {
        id: contentChoice
        anchors.top: parent.top
        anchors.left: menusArea.right
        anchors.bottom: bottomBar.top
        width: parent.width
        visible: module.state == "choiceOfType"
            || module.state == "choiceOfPatient"
        opacity: visible ? 1 : 0

        Behavior on opacity { NumberAnimation { duration: 500 } }

        Rectangle {
            id: choiceTopAreaTitle
            anchors.top: parent.top
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.topMargin: -10
            anchors.leftMargin: 50
            anchors.rightMargin: 50
            height: 70; color: "#5F5F5F"
            border.width: 1; border.color: "grey"
            radius: 6; smooth: true

            CommonText {
                anchors.bottom: typeArea.top
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.bottomMargin: 10
                horizontalAlignment: Text.AlignHCenter
                text: "Résultats des analyses des patients"
            }

            CommonText {
                id: typeArea
                anchors.bottom: parent.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.bottomMargin: 10
                horizontalAlignment: Text.AlignHCenter
                font.pointSize: 11
                text: module.state == "choiceOfType"
                    ? "Choix du type d'analyse"
                    : module.state == "choiceOfPatient"
                    ? "Choix du patient" : ""
            }

            Button {
                anchors.bottom: parent.bottom
                anchors.right: parent.right
                anchors.rightMargin: 15
                anchors.bottomMargin: 18
                visible: module.state == "choiceOfPatient"
                onClicked: module.state = "choiceOfType";
                text: "Retour"; height: 25
                radius: Math.floor(height/2)
                textInput { font.pixelSize: 15 }
            }
        }

        Item {
            anchors.top: choiceTopAreaTitle.bottom
            anchors.bottom: parent.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.topMargin: 20

            ListView {
                anchors.top: parent.top
                anchors.bottom: parent.bottom
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.horizontalCenterOffset: module.state == "choiceOfPatient" ? 0 : 800
                spacing: 20; width: 360; clip: true
                model: module.type == "scans" ? scansModel : radiosModel
                delegate: PatientAnalysisDelegate { }
                opacity: visible ? 1 : 0

                Behavior on anchors.horizontalCenterOffset { SpringAnimation { spring: 2; damping: 0.2 } }
            }

            Column {
                anchors.centerIn: parent
                anchors.verticalCenterOffset: -15
                anchors.horizontalCenterOffset: module.state == "choiceOfType" ? 0 : -800
                spacing: 20

                Behavior on anchors.horizontalCenterOffset { SpringAnimation { spring: 2; damping: 0.2 } }

                Button {
                    id: buttonScan
                    text: "Voir les scans"
                    width: 200
                    textInput {
                        anchors.horizontalCenter: undefined
                        anchors.left: buttonScan.left
                        anchors.leftMargin: 15
                    }
                    onClicked: {
                        module.type = "scans";
                        module.state = "choiceOfPatient"
                    }

                    Image {
                        anchors.right: parent.right
                        anchors.rightMargin: 10
                        anchors.verticalCenter: parent.verticalCenter
                        source: "qrc:/images/right-arrow.png"
                    }
                }

                Button {
                    id: buttonRadios
                    text: "Voir les radios"
                    width: 200
                    textInput {
                        anchors.horizontalCenter: undefined
                        anchors.left: buttonRadios.left
                        anchors.leftMargin: 15
                    }
                    onClicked: {
                        module.type = "radios";
                        module.state = "choiceOfPatient"
                    }

                    Image {
                        anchors.right: parent.right
                        anchors.rightMargin: 10
                        anchors.verticalCenter: parent.verticalCenter
                        source: "qrc:/images/right-arrow.png"
                    }
                }
            }
        }
    }

    Item {
        id: contentScans
        anchors.left: contentChoice.left
        anchors.right: contentChoice.right
        anchors.top: contentChoice.top
        anchors.bottom: contentChoice.bottom
        visible: module.state == "viewScans"
        opacity: visible ? 1 : 0

        Behavior on opacity { NumberAnimation { duration: 500 } }

        Item {
            id: scansLeftArea
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.bottom: parent.bottom
            anchors.leftMargin: 10
            width: 255

            Rectangle {
                id: scansLeftAreaTitle
                anchors.top: parent.top
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.topMargin: -10
                height: 45; color: "#5F5F5F"
                border.width: 1; border.color: "grey"
                radius: 6; smooth: true

                Row {
                    anchors.bottom: parent.bottom
                    anchors.left: parent.left
                    anchors.leftMargin: 15
                    anchors.bottomMargin: 5
                    spacing: 15

                    Button {
                        onClicked: module.state = "choiceOfPatient";
                        text: "Retour"; height: 25
                        radius: Math.floor(height/2)
                        textInput { font.pixelSize: 15 }
                    }

                    CommonText {
                        text: "Miniatures"; height: 25
                        verticalAlignment: Text.AlignVCenter
                    }
                }
            }

            Rectangle {
                id: scansLeftAreaContent
                anchors.top: scansLeftAreaTitle.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.bottom: parent.bottom
                anchors.bottomMargin: -bottomBar.height - 10
                anchors.topMargin: 10; radius: 6
                color: "#5F5F5F"; smooth: true
                border.width: 1; border.color: "grey"

                GridView {
                    opacity: visible ? 1 : 0
                    anchors.fill: parent
                    anchors.leftMargin: 5
                    anchors.rightMargin: 5
                    cellWidth: Math.floor(width/2)
                    cellHeight: 125; clip: true
                    cacheBuffer: contentHeight
                    anchors.bottomMargin: bottomBar.height + 10
                    delegate: AnalysisThumbnailDelegate {
                        MouseArea {
                            anchors.fill: parent
                            onClicked: { module.scanCurrentlyViewed = index; }
                        }
                    }
                    model: scanPaths
                }
            }
        }

        Rectangle {
            id: scansTopRightArea
            anchors.left: scansLeftArea.right
            anchors.right: parent.right
            anchors.top: parent.top
            anchors.leftMargin: 11
            anchors.rightMargin: 10
            anchors.topMargin: -10
            height: 65; color: "#5F5F5F"
            border.width: 1; border.color: "grey"
            radius: 6; smooth: true


            CommonText {
                id: scanOwnerArea
                anchors.bottom: scanBodyPartArea.top
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.bottomMargin: 5
                horizontalAlignment: Text.AlignHCenter
                text: "Scans de "
                    + (patientsModel.count > module.patient
                    && patientsModel.get(module.patient)
                    ? patientsModel.get(module.patient).name
                    : "")
            }

            CommonText {
                id: scanBodyPartArea
                anchors.bottom: parent.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.bottomMargin: 10
                horizontalAlignment: Text.AlignHCenter
                font.pointSize: 11
                opacity: 0.7
                onVisibleChanged: {
                    var scan = scansModel.get(module.patientCurrentlyViewed);
                    if (visible && scan) {
                        var bodyPart = bodyPartsModel.get(scan.group_id);
                        if (bodyPart) {
                            text = "Partie : " + bodyPart.name;
                            return;
                        }
                    }
                    text = "Partie du corps indéterminée";
                }
            }
        }

        Image {
            anchors.right: scanPictureArea.left
            anchors.verticalCenter: scanPictureArea.verticalCenter
            anchors.rightMargin: 10
            source: "qrc:/images/left-arrow.png"
            opacity: module.scanCurrentlyViewed <= 0 ? 0.5 : 1

            Behavior on opacity { NumberAnimation { duration: 250 } }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    module.scanCurrentlyViewed = Math.max(0,
                        module.scanCurrentlyViewed-1);
                }
            }
        }

        Image {
            anchors.left: scanPictureArea.right
            anchors.verticalCenter: scanPictureArea.verticalCenter
            anchors.leftMargin: 10
            source: "qrc:/images/right-arrow.png"
            opacity: module.scanCurrentlyViewed >= scanPaths.count-1 ? 0.5 : 1

            Behavior on opacity { NumberAnimation { duration: 250 } }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    module.scanCurrentlyViewed = Math.min(scanPaths.count-1,
                        module.scanCurrentlyViewed+1);
                }
            }
        }

        Image {
            id: scanPictureArea
            anchors.left: scansTopRightArea.left
            anchors.right: scansTopRightArea.right
            anchors.top: scansTopRightArea.bottom
            anchors.bottom: parent.bottom
            anchors.topMargin: 10
            anchors.bottomMargin: 10
            anchors.leftMargin: 30
            anchors.rightMargin: 30
            fillMode: Image.PreserveAspectCrop
            smooth: true; clip: true
            source: scanPaths.count > module.scanCurrentlyViewed
                && scanPaths.get(module.scanCurrentlyViewed)
                ? scanPaths.get(module.scanCurrentlyViewed).path
                : ""
        }
    }

    Rectangle {
        id: fullscreenScanArea
        anchors.fill: parent
        color: "#000000"; z: 2
        visible: module.fullscreenScansVisible
        opacity: visible ? 1 : 0

        onVisibleChanged: runningTimer.running = visible;
        property bool actionBarVisible: false

        MouseArea {
            anchors.fill: parent
        }

        Behavior on opacity { NumberAnimation { duration: 250 } }

        Rectangle {
            id: fullscreenScanTitle
            anchors.top: parent.top
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.topMargin: -10
            anchors.leftMargin: 50
            anchors.rightMargin: 50
            height: 70; color: "#5F5F5F"
            border.width: 1; border.color: "grey"
            radius: 6; smooth: true

            CommonText {
                anchors.bottom: scanTypeArea.top
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.bottomMargin: 10
                horizontalAlignment: Text.AlignHCenter
                text: scanOwnerArea.text
            }

            CommonText {
                id: scanTypeArea
                anchors.bottom: parent.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.bottomMargin: 10
                horizontalAlignment: Text.AlignHCenter
                font.pointSize: 11
                opacity: 0.7
                text: scanBodyPartArea.text
            }

            Button {
                anchors.bottom: parent.bottom
                anchors.right: parent.right
                anchors.rightMargin: 15
                anchors.bottomMargin: 18
                onClicked: module.fullscreenScansVisible = false;
                text: "Retour"; height: 25
                radius: Math.floor(height/2)
                textInput { font.pixelSize: 15 }
            }
        }

        Item {
            id: scansRepeater; clip: true
            anchors.top: fullscreenScanTitle.bottom
            anchors.bottom: parent.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.topMargin: 5

            Repeater {
                model: scanPaths
                delegate: Item {
                    visible: module.scanCurrentlyViewed == index
                    height: scansRepeater.height
                    width: scansRepeater.width
                    Image {
                        height: parent.height
                        anchors.horizontalCenter: parent.horizontalCenter
                        fillMode: Image.PreserveAspectFit
                        smooth: true; source: path
                    }
                }
            }

            Timer {
                id: runningTimer
                running: false
                interval: 75
                repeat: true
                onTriggered: {
                    var value = module.scanCurrentlyViewed;
                    value++;
                    if (value >= scanPaths.count)
                        value = 0;
                    module.scanCurrentlyViewed = value;
                }
            }
        }

        Item {
            anchors.top: parent.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.leftMargin: 50
            anchors.rightMargin: 50
            anchors.topMargin: fullscreenScanArea.actionBarVisible
                ? -75 : 0
            height: 100
            z: 1

            Behavior on anchors.topMargin { SpringAnimation { spring: 2; damping: 0.2 } }

            Rectangle {
                anchors.fill: parent
                opacity: 0.3
                radius: 5; smooth: true
                z: -1
            }

            Button {
                anchors.right: actionPlayPause.left
                anchors.rightMargin: 15
                anchors.verticalCenter: actionPlayPause.verticalCenter
                onClicked: runningTimer.interval += 25;
                text: "Vitesse -"; height: 25
                radius: Math.floor(height/2)
                textInput { font.pixelSize: 15 }
                opacity: 0.8
            }

            Button {
                id: actionPlayPause
                anchors.top: parent.top
                anchors.topMargin: 15
                height: 45; width: 75
                radius: 20; smooth: true
                anchors.horizontalCenter: parent.horizontalCenter
                textInput { font.pointSize: 15 }
                text: runningTimer.running ? "Pause" : "Play"
                onClicked: runningTimer.running = !runningTimer.running;
                opacity: 0.8
            }

            Button {
                anchors.left: actionPlayPause.right
                anchors.leftMargin: 15
                anchors.verticalCenter: actionPlayPause.verticalCenter
                onClicked: runningTimer.interval = Math.max(25, runningTimer.interval-25);
                text: "Vitesse +"; height: 25
                radius: Math.floor(height/2)
                textInput { font.pixelSize: 15 }
                opacity: 0.8
            }
        }

        Timer {
            id: disappearTimer
            interval: 1500
            repeat: false
            onTriggered: {
                fullscreenScanArea.actionBarVisible = false;
            }
        }

        MouseArea {
            z: 2
            anchors.fill: parent
            hoverEnabled: true
            onPositionChanged: {
                parent.actionBarVisible = true;
                disappearTimer.restart();
                mouse.accepted = false;
            }
            onPressed: {
                parent.actionBarVisible = true;
                disappearTimer.restart();
                mouse.accepted = false;
            }
            onReleased: mouse.accepted = false;
        }
    }

    Item {
        id: contentRadios
        anchors.left: contentChoice.left
        anchors.right: contentChoice.right
        anchors.top: contentChoice.top
        anchors.bottom: contentChoice.bottom
        visible: module.state == "viewRadios"
        opacity: visible ? 1 : 0

        MouseArea {
            anchors.fill: parent
        }

        Behavior on opacity { NumberAnimation { duration: 500 } }

        Flickable {
            anchors.fill: parent
            contentHeight: radioGrid.zoomLevel != 4 ? radioGrid.height : height
            contentWidth: radioGrid.zoomLevel != 4 ? radioGrid.width : width

            Grid {
                id: radioGrid

                property int zoomLevel: 4

                width: childrenRect.width
                height: childrenRect.height
                anchors.centerIn: parent
                columns: radiosModel.get(module.patientCurrentlyViewed) ?
                    radiosModel.get(module.patientCurrentlyViewed).y_parts : 10
                rows: radiosModel.get(module.patientCurrentlyViewed) ?
                    radiosModel.get(module.patientCurrentlyViewed).x_parts : 10

                Repeater {
                    model: radioPaths
                    delegate: Image {
                        source: path
                        width: radioGrid.zoomLevel != 4 ? sourceSize.width / radioGrid.zoomLevel
                            : (contentRadios.width/radioGrid.columns
                            > contentRadios.height/radioGrid.rows
                            ? contentRadios.height/radioGrid.rows
                            : contentRadios.width/radioGrid.columns)
                        height: radioGrid.zoomLevel != 4 ? sourceSize.height / radioGrid.zoomLevel
                            : (contentRadios.height/radioGrid.rows
                            > contentRadios.width/radioGrid.columns
                            ? contentRadios.width/radioGrid.columns
                            : contentRadios.height/radioGrid.rows)
                    }
                }
            }
        }

        Rectangle {
            anchors.right: parent.right
            anchors.top: parent.top
            anchors.rightMargin: 25
            anchors.topMargin: 25; radius: 5
            width: actionZoomColumn.width + 50
            height: actionZoomColumn.height + 50
            color: "#5F5F5F"; smooth: true
            border.width: 1; border.color: "grey"

            Column {
                id: actionZoomColumn
                anchors.centerIn: parent
                width: childrenRect.width
                height: childrenRect.height
                spacing: 10

                Button {
                    width: 35; height: 35
                    text: "+"; radius: Math.floor(height/2)
                    textInput { font.pointSize: 15 }
                    onClicked: radioGrid.zoomLevel = Math.max(1, radioGrid.zoomLevel-1);
                    opacity: radioGrid.zoomLevel <= 1 ? 0.5 : 1
                }

                Button {
                    width: 35; height: 35
                    text: "-"; radius: Math.floor(height/2)
                    textInput { font.pointSize: 15 }
                    onClicked: radioGrid.zoomLevel = Math.min(4, radioGrid.zoomLevel+1);
                    opacity: radioGrid.zoomLevel >= 4 ? 0.5 : 1
                }
            }
        }
    }

    BottomBar {
        id: bottomBar

        Button {
            text: "Animer"
            border.color: "#424242"
            anchors.right: parent.right
            anchors.rightMargin: 10
            anchors.verticalCenter: parent.verticalCenter
            visible: module.state == "viewScans"
            onClicked: {
                module.fullscreenScansVisible = true;
            }
        }

        Button {
            id: buttonDetails
            text: "Détails"
            border.color: "#424242"
            anchors.right: parent.right
            anchors.rightMargin: 10
            anchors.verticalCenter: parent.verticalCenter
            visible: module.state == "viewRadios"
            onClicked: module.popupRadioInfosVisible = true;
        }

        Button {
            text: "Retour"
            border.color: "#424242"
            anchors.right: buttonDetails.left
            anchors.rightMargin: 10
            anchors.verticalCenter: parent.verticalCenter
            visible: module.state == "viewRadios"
            onClicked: module.state = "choiceOfPatient";
        }
    }
}
