/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

import QtQuick 1.0
import "Components"
import "Delegates"
import "Gradients"
import "Popups"

Rectangle {
    id: module
    anchors.fill: parent
    color: "#545454"

    property bool popupPatientInfosVisible: false
    property bool popupSymptomOrDiagnosticVisible: false
    property bool popupPrescriptionVisible: false
    property bool popupConfirmVisible: false

    function loadPatientInfos(patientId) {
        anchor.requestForPatient(patientId);
    }
    function modifyPatient() {
        popupPatientInfos.patientId = main.patientCurrentlyViewed;
        popupPatientInfosVisible = true;
    }
    function newPatient() {
        popupPatientInfos.patientId = -1;
        popupPatientInfosVisible = true;
    }
    function addSymptom() {
        popupSymptomOrDiagnostic.isSymptom = true;
        popupSymptomOrDiagnosticVisible = true;
    }
    function addDiagnostic() {
        popupSymptomOrDiagnostic.isSymptom = false;
        popupSymptomOrDiagnosticVisible = true;
    }
    function addPrescription() {
        popupPrescription.medicineId = -1;
        popupPrescription.prescriptionId = -1;
        popupPrescriptionVisible = true;
    }
    function modifyPrescription(index, prescriptionId) {
        popupPrescription.medicineId = prescriptionId;
        popupPrescription.prescriptionId = index;
        popupPrescriptionVisible = true;
    }
    function removePrescription(index) {
        popupConfirm.prescriptionId = index;
        popupConfirmVisible = true;
    }

    Background { }

    PopupPatientInfos {
        id: popupPatientInfos
        visible: popupPatientInfosVisible
        opacity: visible ? 1 : 0
        onHideRequest: {
            if (accepted) {
                main.modifyOrAddPatient(patientId, name, birthyear,
                    man, address, alive, canGoHome, urgencied);
            }
            popupPatientInfosVisible = false;
        }
    }

    PopupSymptomOrDiagnostic {
        id: popupSymptomOrDiagnostic
        visible: popupSymptomOrDiagnosticVisible
        opacity: visible ? 1 : 0
        onHideRequest: {
            if (accepted) {
                if (isSymptom)
                    anchor.newSymptom(main.patientCurrentlyViewed, text);
                else
                    anchor.newDiagnostic(main.patientCurrentlyViewed, text);
            }
            popupSymptomOrDiagnosticVisible = false;
        }
    }

    PopupQuestion {

        property int prescriptionId: -1

        id: popupConfirm
        title: "Confirmer la suppression ?"
        visible: popupConfirmVisible
        opacity: visible ? 1 : 0
        onHideRequest: {
            if (accepted && prescriptionId != -1) {
                anchor.removePrescription(main.patientCurrentlyViewed,
                    prescriptionId);
            }
            popupConfirmVisible = false;
        }
    }

    PopupPrescriptions {
        id: popupPrescription
        visible: popupPrescriptionVisible
        opacity: visible ? 1 : 0
        onHideRequest: {
            if (accepted) {
                if (popupPrescription.medicineId < 0) {
                    anchor.newPrescription(main.patientCurrentlyViewed,
                        duration, timestamp, medicine_id);
                }
                else {
                    anchor.modifyPrescription(main.patientCurrentlyViewed,
                        popupPrescription.medicineId, duration, timestamp,
                        medicine_id);
                }
            }
            popupPrescriptionVisible = false;
        }
    }

    MouseArea {
        anchors.fill: parent
        z: 15
        onPressed: {
            // Retrait du focus du champ de recherche lorsqu'on clique à l'extérieur
            var pos = searchField.mapToItem(module, 0, 0);
            if ((mouse.x < pos.x || mouse.x > pos.x + searchField.width)
                || (mouse.y < pos.y || mouse.y > pos.y + searchField.height))
                searchField.textInput.focus = false;
            mouse.accepted = false;
        }
    }

    MenuAccessor { }

    Menus {
        id: menusArea
        contentHeight: content.height
    }

    Item {
        id: content
        anchors.top: parent.top
        anchors.left: menusArea.right
        anchors.bottom: bottomBar.top
        width: parent.width

        Item {
            id: leftArea
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.bottom: parent.bottom
            anchors.leftMargin: 10
            width: 225

            Rectangle {
                id: leftAreaTitle
                anchors.top: parent.top
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.topMargin: -10
                height: 45; color: "#5F5F5F"
                border.width: 1; border.color: "grey"
                radius: 6; smooth: true

                CommonText {
                    anchors.bottom: parent.bottom
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.bottomMargin: 10
                    horizontalAlignment: Text.AlignHCenter
                    text: "Liste des patients"
                }
            }

            Rectangle {
                id: leftAreaContent
                anchors.top: leftAreaTitle.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.bottom: parent.bottom
                anchors.bottomMargin: -bottomBar.height - 10
                anchors.topMargin: 10; radius: 6
                color: "#5F5F5F"; smooth: true
                border.width: 1; border.color: "grey"

                CommonTextInput {
                    id: searchField
                    anchors.left: parent.left
                    anchors.top: parent.top
                    anchors.right: parent.right
                    anchors.topMargin: 10
                    anchors.leftMargin: 10
                    height: 30

                    onEditingFinished: textInput.focus = false;

                    CommonText {
                        opacity: parent.textInput.text != ""
                            || parent.textInput.focus ? 0 : 0.3
                        text: "Rechercher..."
                        anchors.fill: parent
                        anchors.margins: 4
                        color: "#000000"
                        font.pointSize: 12
                        style: Text.Normal
                    }
                }

                Separator {
                    id: searchSeparator
                    anchors.top: searchField.bottom
                    anchors.topMargin: 10
                    anchors.left: parent.left
                    anchors.right: parent.right
                }

                ListView {
                    id: patientsListView; clip: true
                    anchors.top: searchSeparator.bottom
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.bottom: parent.bottom
                    anchors.topMargin: -1
                    anchors.bottomMargin: bottomBar.height + 10
                    model: patientsModel
                    delegate: PatientsListDelegate { }
                }
            }
        }

        Rectangle {
            id: topRightArea
            anchors.left: leftArea.right
            anchors.right: parent.right
            anchors.top: parent.top
            anchors.leftMargin: 11
            anchors.rightMargin: 10
            anchors.topMargin: -10
            height: 85; color: "#5F5F5F"
            border.width: 1; border.color: "grey"
            radius: 6; smooth: true
            visible: patientInfos.count == 1

            Image {
                id: deadStateArea
                source: "qrc:/images/polaroid.png"
                width: 51; height: 65
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 5
                anchors.left: parent.left
                anchors.leftMargin: 5

                Image {
                    height: 36; clip: true
                    fillMode: Image.PreserveAspectCrop
                    source: patientInfos.count == 1
                        && patientInfos.get(0).isDead == 1
                        ? "qrc:/images/dead.png"
                        : "qrc:/images/alive.png"
                    anchors.left: parent.left
                    anchors.top: parent.top
                    anchors.right: parent.right
                    anchors.leftMargin: 5
                    anchors.rightMargin: 5
                    anchors.topMargin: 6
                    smooth: true; z: -1
                }

                CommonText {
                    style: Text.Normal
                    font.pixelSize: 14
                    text: patientInfos.count == 1
                        && patientInfos.get(0).isDead == 1
                        ? "Mort"
                        : "Vivant"
                    font.bold: false
                    color: "#000000"
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: 5
                    anchors.horizontalCenter: parent.horizontalCenter
                }
            }

            CommonText {
                id: nameArea
                anchors.left: deadStateArea.right
                anchors.right: genderArea.left
                anchors.top: parent.top
                anchors.leftMargin: 10
                anchors.topMargin: 18
                anchors.rightMargin: 10
                height: 25; elide: Text.ElideRight
                verticalAlignment: Text.AlignVCenter
                text: patientInfos.count < 1 ? "" : patientInfos.get(0).name
            }

            CommonText {
                id: genderArea
                anchors.right: parent.right
                anchors.top: nameArea.top
                anchors.bottom: nameArea.bottom
                anchors.rightMargin: 10
                text: (patientInfos.count < 1 ? ""
                    : (patientInfos.get(0).age + " an"
                    + (patientInfos.get(0).age > 1 ? "s" : "") + " - "))
                    + (patientInfos.count < 1 ? ""
                    : patientInfos.get(0).gender == 1
                    ? "Femme" : "Homme")
            }

            Button {
                id: modifyPatientButton
                anchors.right: genderArea.right
                anchors.top: genderArea.bottom
                anchors.topMargin: 10
                text: "Modifier"; height: 25
                radius: Math.floor(height/2)
                textInput { font.pixelSize: 15 }
                onClicked: module.modifyPatient()
            }

            CommonText {
                anchors.left: deadStateArea.right
                anchors.right: modifyPatientButton.left
                anchors.top: nameArea.bottom
                anchors.leftMargin: 10
                anchors.topMargin: 10
                anchors.rightMargin: 10
                height: 25; elide: Text.ElideRight
                verticalAlignment: Text.AlignVCenter
                text: patientInfos.count < 1 ? ""
                    : patientInfos.get(0).urgencied == 1
                    ? "En soins intensifs"
                    : patientInfos.get(0).canGoHome == 1
                    ? "Prêt" + (patientInfos.get(0).gender == 1 ? "e" : "")
                    + " à sortir de l'hôpital"
                    : "Interné" + (patientInfos.get(0).gender == 1 ? "e" : "")
            }
        }

        TabWidget {
            clip: true
            anchors.top: topRightArea.bottom
            anchors.left: topRightArea.left
            anchors.right: topRightArea.right
            anchors.bottom: parent.bottom
            anchors.bottomMargin: -bottomBar.height - 10
            anchors.topMargin: 10; color: "#5F5F5F"
            border.width: 1; border.color: "grey"

            Item {
                property string title: "Symptômes"
                anchors.fill: parent
                anchors.bottomMargin: bottomBar.height + 10

                Button {
                    id: addSymtomButton
                    text: "Nouveau symptôme"
                    anchors.top: parent.top
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.margins: 10
                    onClicked: module.addSymptom();
                }

                ListView {
                    id: symptomsArea; clip: true
                    anchors.top: addSymtomButton.bottom
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.bottom: parent.bottom
                    anchors.topMargin: 10
                    model: patientSymptoms
                    delegate: PatientDelegate {
                        width: symptomsArea.width
                        _date: date
                        _text: symptom
                    }

                    CommonText {
                        anchors.fill: parent
                        anchors.leftMargin: 10
                        visible: parent.model.count == 0
                        text: "Aucun symptôme..."
                        font.italic: true
                    }
                }
            }
            Item {
                property string title: "Diagnostics"
                anchors.fill: parent
                anchors.bottomMargin: bottomBar.height + 10

                Button {
                    id: addDiagnosticButton
                    text: "Nouveau diagnostic"
                    anchors.top: parent.top
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.margins: 10
                    onClicked: module.addDiagnostic();
                }

                ListView {
                    id: diagnosticsArea; clip: true
                    anchors.top: addDiagnosticButton.bottom
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.bottom: parent.bottom
                    anchors.topMargin: 10
                    model: patientDiagnostics
                    delegate: PatientDelegate {
                        width: diagnosticsArea.width
                        _date: date
                        _text: diagnostic
                    }

                    CommonText {
                        anchors.fill: parent
                        anchors.leftMargin: 10
                        visible: parent.model.count == 0
                        text: "Aucun diagnostic..."
                        font.italic: true
                    }
                }
            }
            Item {
                property string title: "Prescriptions"
                anchors.fill: parent
                anchors.bottomMargin: bottomBar.height + 10

                Button {
                    id: addPrescriptionButton
                    text: "Nouvelle prescription"
                    anchors.top: parent.top
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.margins: 10
                    onClicked: module.addPrescription();
                }

                ListView {
                    id: prescriptionsArea; clip: true
                    anchors.top: addPrescriptionButton.bottom
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.bottom: parent.bottom
                    anchors.topMargin: 10
                    model: patientPrescriptions
                    delegate: PatientPrescriptionDelegate { }

                    CommonText {
                        anchors.fill: parent
                        anchors.leftMargin: 10
                        visible: parent.model.count == 0
                        text: "Aucune prescription..."
                        font.italic: true
                    }
                }
            }
        }
    }

    BottomBar {
        id: bottomBar

        Button {
            id: btnInfos
            text: "Nouveau patient"
            border.color: "#424242"
            anchors.right: parent.right
            anchors.rightMargin: 10
            anchors.verticalCenter: parent.verticalCenter
            onClicked: module.newPatient();
        }
    }
}
