/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

import QtQuick 1.0
import Utopital 1.0
import "Components"
import "Delegates"
import "Gradients"
import "Popups"

Rectangle {
    id: module
    anchors.fill: parent
    color: "#545454"

    property int patientCurrentlyViewed: -1
    property int viewerInterval: 3
    property int viewerDuration: 3
    property int viewerPWidth: 3
    property color viewerColor: "#56B058"
    property bool popupColorVisible: false

    onViewerIntervalChanged: ecgViewer.yInterval = viewerInterval;
    onViewerDurationChanged: ecgViewer.duration = viewerDuration;
    onViewerPWidthChanged: ecgViewer.pWidth = viewerPWidth;
    onViewerColorChanged: ecgViewer.color = viewerColor;
    onVisibleChanged: {
        if (!visible) {
            content.flipped = false;
            ecgViewer.clear();
            anchor.stopECG();
        }
    }

    function viewECG(index) {
        waitForStart.start();
        patientCurrentlyViewed = index;
        content.flipped = true;
    }

    function handlePoints(data) {
        for (var i = 0; i < data.length; i++)
            ecgViewer.addPoint(data[i]);
        ecgViewer.refresh();
    }

    Timer {
        id: waitForStart
        interval: 2000
        repeat: false
        running: false
        onTriggered: {
            anchor.startECG(parent.patientCurrentlyViewed);
        }
    }

    PopupColorPicker {
        visible: module.popupColorVisible
        opacity: visible ? 1 : 0
        title: "Sélection de couleur"
        onHideRequest: {
            if (accepted)
                ecgViewer.color = finalColor;
            module.popupColorVisible = false;
        }
    }

    Background { }

    MenuAccessor { }

    Menus {
        id: menusArea
        contentHeight: content.height
    }

    FlipableFrame {
        id: content
        anchors.top: parent.top
        anchors.left: menusArea.right
        anchors.bottom: bottomBar.top
        width: parent.width

        front: Item {
            anchors.fill: parent

            Rectangle {
                id: frontTopAreaTitle
                anchors.top: parent.top
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.topMargin: -10
                anchors.leftMargin: 50
                anchors.rightMargin: 50
                height: 45; color: "#5F5F5F"
                border.width: 1; border.color: "grey"
                radius: 6; smooth: true

                CommonText {
                    anchors.bottom: parent.bottom
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.bottomMargin: 10
                    horizontalAlignment: Text.AlignHCenter
                    text: "Liste des patients aux urgences"
                }
            }

            ListView {
                anchors.top: frontTopAreaTitle.bottom
                anchors.bottom: parent.bottom
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.topMargin: 20; spacing: 20
                width: 360; clip: true
                model: patientUrgenciedModel
                delegate: PatientUrgenciedDelegate { }
            }
        }

        back: Item {
            anchors.fill: parent

            ECGViewer {
                id: ecgViewer
                anchors.top: parent.top
                anchors.left: parent.left
                anchors.right: rightAreaTitle.left
                anchors.bottom: parent.bottom

                Component.onCompleted: {
                    yInterval = module.viewerInterval;
                    duration = module.viewerDuration;
                    pWidth = module.viewerPWidth;
                    color = module.viewerColor;
                }
            }

            Rectangle {
                id: rightAreaTitle
                anchors.top: parent.top
                anchors.right: parent.right
                anchors.topMargin: -10
                anchors.rightMargin: 10
                width: 250; height: 45
                color: "#5F5F5F"
                border.width: 1; border.color: "grey"
                radius: 6; smooth: true

                CommonText {
                    anchors.bottom: parent.bottom
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.bottomMargin: 10
                    anchors.leftMargin: 10
                    anchors.rightMargin: 10
                    horizontalAlignment: Text.AlignHCenter
                    text: "ECG de " + (patientUrgenciedModel.count
                        > module.patientCurrentlyViewed
                        && patientUrgenciedModel.get(module.patientCurrentlyViewed)
                        ? patientUrgenciedModel.get(module.patientCurrentlyViewed).name
                        : "")
                    elide: Text.ElideRight
                }
            }

            Rectangle {
                anchors.top: rightAreaTitle.bottom
                anchors.right: parent.right
                anchors.bottom: parent.bottom
                anchors.bottomMargin: -bottomBar.height - 10
                anchors.topMargin: 10
                anchors.rightMargin: 10
                width: 250; radius: 6
                color: "#5F5F5F"; smooth: true
                border.width: 1; border.color: "grey"

                Item {
                    anchors.top: parent.top
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.bottom: parent.bottom
                    anchors.topMargin: 10
                    anchors.bottomMargin: bottomBar.height + 10

                    CommonText {
                        id: colorLabel
                        anchors.left: parent.left
                        anchors.leftMargin: 10
                        text: "Couleur du trait :"; height: 25
                        verticalAlignment: Text.AlignVCenter
                    }

                    Button {
                        anchors.left: colorLabel.right
                        anchors.right: parent.right
                        anchors.leftMargin: 10
                        anchors.rightMargin: 10
                        text: "Sélectionner..."
                        height: 25
                        textInput { font.pixelSize: 14 }
                        radius: Math.floor(height/2)
                        onClicked: {
                            module.popupColorVisible = true;
                        }
                    }

                    CommonText {
                        id: painterSizeLabel
                        height: 25
                        anchors.top: colorLabel.bottom
                        anchors.topMargin: 10
                        anchors.left: parent.left
                        anchors.leftMargin: 10
                        text: "Épaisseur du trait :"
                        verticalAlignment: Text.AlignVCenter
                    }

                    CommonTextInput {
                        height: 25
                        anchors.left: painterSizeLabel.right
                        anchors.right: painterSizeEndLabel.left
                        anchors.top: colorLabel.bottom
                        anchors.leftMargin: 5
                        anchors.rightMargin: 5
                        anchors.topMargin: 10
                        textInput {
                            text: module.viewerPWidth
                            validator: IntValidator { bottom: 1; top: 10 }
                            onTextChanged: {
                                module.viewerPWidth = parseInt(text);
                            }
                        }
                    }

                    CommonText {
                        id: painterSizeEndLabel
                        height: 25
                        anchors.right: parent.right
                        anchors.top: colorLabel.bottom
                        anchors.rightMargin: 10
                        anchors.topMargin: 10
                        text: "px"
                        verticalAlignment: Text.AlignVCenter
                    }

                    Separator {
                        id: painterLookAndFeedSeparator
                        anchors.top: painterSizeLabel.bottom
                        anchors.left: parent.left
                        anchors.right: parent.right
                        anchors.topMargin: 10
                    }

                    CommonText {
                        id: yIntervalLabel
                        height: 25
                        anchors.top: painterLookAndFeedSeparator.bottom
                        anchors.topMargin: 10
                        anchors.left: parent.left
                        anchors.leftMargin: 10
                        text: "Écart d'amplitude :"
                        verticalAlignment: Text.AlignVCenter
                    }

                    CommonTextInput {
                        height: 25
                        anchors.left: yIntervalLabel.right
                        anchors.right: yIntervalEndLabel.left
                        anchors.top: painterLookAndFeedSeparator.bottom
                        anchors.leftMargin: 5
                        anchors.rightMargin: 5
                        anchors.topMargin: 10
                        textInput {
                            text: module.viewerInterval
                            validator: IntValidator { bottom: 1; top: 15 }
                            onTextChanged: {
                                module.viewerInterval = parseInt(text);
                            }
                        }
                    }

                    CommonText {
                        id: yIntervalEndLabel
                        height: 25
                        anchors.right: parent.right
                        anchors.top: painterLookAndFeedSeparator.bottom
                        anchors.rightMargin: 10
                        anchors.topMargin: 10
                        text: "mV"
                        verticalAlignment: Text.AlignVCenter
                    }

                    CommonText {
                        id: durationLabel
                        height: 25
                        anchors.top: yIntervalLabel.bottom
                        anchors.topMargin: 10
                        anchors.left: parent.left
                        anchors.leftMargin: 10
                        text: "Durée de la capture :"
                        verticalAlignment: Text.AlignVCenter
                    }

                    CommonTextInput {
                        height: 25
                        anchors.left: durationLabel.right
                        anchors.right: durationEndLabel.left
                        anchors.top: yIntervalLabel.bottom
                        anchors.leftMargin: 5
                        anchors.rightMargin: 5
                        anchors.topMargin: 10
                        textInput {
                            text: module.viewerDuration
                            validator: IntValidator { bottom: 1; top: 15 }
                            onTextChanged: {
                                module.viewerDuration = parseInt(text);
                            }
                        }
                    }

                    CommonText {
                        id: durationEndLabel
                        height: 25
                        anchors.right: parent.right
                        anchors.top: yIntervalLabel.bottom
                        anchors.rightMargin: 10
                        anchors.topMargin: 10
                        text: "sec"
                        verticalAlignment: Text.AlignVCenter
                    }
                }
            }
        }
    }

    BottomBar {
        id: bottomBar
    }
}
