/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

import QtQuick 1.0
import "../Components"

MouseArea {

    signal hideRequest(bool accepted, string duration, int timestamp, int medicine_id);

    // Si medicineId < 0, la popup gère l'ajout d'une prescription
    // Sinon, modification de la prescription medicineId.
    property int medicineId: -1
    property int prescriptionId: -1
    property int selectedMedicineId: -1
    property bool popupInfosVisible: false
    property bool popupSelectionVisible: false

    // "private"
    signal finished(bool accepted)

    onFinished: {
        if (accepted) {
            if (selectedMedicineId < 0) {
                popupInfos.text = "Vous devez choisir un médicament.";
                popupInfosVisible = true;
                return;
            }
            if (durationField.text == "") {
                popupInfos.text = "Vous devez renseigner la durée du traitement.";
                popupInfosVisible = true;
                return;
            }
        }
        hideRequest(accepted, durationField.text,
            new Date().getTime()/1000, selectedMedicineId);
    }

    onVisibleChanged: {
        if (visible) {
            var medicine = patientPrescriptions.get(prescriptionId);
            if (medicine) {
                durationField.text = medicine.duration;
                selectedMedicineId = medicine.medicine_id;
            }
            else {
                durationField.text = "";
                selectedMedicineId = -1;
            }
        }
    }

    id: popup
    anchors.fill: parent; z: 2
    hoverEnabled: true

    PopupInformation {
        id: popupInfos
        visible: popupInfosVisible
        opacity: visible ? 1 : 0
        title: "Erreur"; z: 4
        onHideRequest: popupInfosVisible = false;
    }

    PopupSelectMedicine {
        id: popupSelection
        visible: popupSelectionVisible
        opacity: visible ? 1 : 0
        title: "Sélectionnez un médicament"
        onHideRequest: {
            if (accepted)
                selectedMedicineId = index;
            popupSelectionVisible = false;
        }
    }

    Rectangle {
        anchors.fill: parent
        color: "#000000"
        opacity: 0.7
    }

    ComplexPopup {
        width: 475
        height: 185
        anchors.centerIn: parent
        anchors.verticalCenterOffset: - Math.floor(bottomBar.height/2)
        onHideRequest: popup.finished(false);
        opacity: parent.visible ? 1 : 0
        title: !patientPrescriptions.get(medicineId)
            ? "Nouvelle prescription"
            : "Modification d'une prescription"

        Behavior on opacity { NumberAnimation { duration: 250 } }

        Column {
            id: contentArea
            anchors.top: parent.top
            anchors.topMargin: 10
            width: parent.width
            height: childrenRect.height
            spacing: 10

            Item {
                width: parent.width
                height: 30

                CommonText {
                    id: durationLabel
                    text: "Durée :"
                    anchors.left: parent.left
                    anchors.verticalCenter: parent.verticalCenter
                }
                CommonTextInput {
                    id: durationField
                    height: 25
                    anchors.right: parent.right
                    anchors.left: durationLabel.right
                    anchors.leftMargin: 10
                    anchors.verticalCenter: parent.verticalCenter
                }
            }

            Item {
                width: parent.width
                height: 30

                CommonText {
                    id: medicineLabel
                    text: "Médicament :"
                    anchors.verticalCenter: parent.verticalCenter
                }
                Rectangle {
                    color: "#885F5F5F"
                    border.width: 1; height: 30
                    border.color: "#424242"
                    anchors.left: medicineLabel.right
                    anchors.right: medicineButton.left
                    anchors.leftMargin: 10
                    anchors.rightMargin: 10
                    anchors.verticalCenter: parent.verticalCenter

                    CommonText {
                        text: medicineModel.get(selectedMedicineId)
                            ? medicineModel.get(selectedMedicineId).name : ""
                        anchors.left: parent.left
                        anchors.right: parent.right
                        anchors.leftMargin: 10
                        anchors.rightMargin: 10
                        anchors.verticalCenter: parent.verticalCenter
                        elide: Text.ElideRight
                    }
                }
                Button {
                    id: medicineButton
                    text: "Choisir un médicament..."
                    textInput { font.pixelSize: 14 }
                    height: 25
                    anchors.right: parent.right
                    border.color: "#424242"
                    anchors.verticalCenter: parent.verticalCenter
                    radius: Math.floor(height/2)
                    onClicked: {
                        popupSelection.currentMedicine
                            = selectedMedicineId == -1 ? 0 : selectedMedicineId
                        popup.popupSelectionVisible = true;
                    }
                }
            }
        }

        Rectangle {
            id: bottomArea
            width: 175
            anchors.right: parent.right
            anchors.bottom: parent.bottom
            anchors.bottomMargin: -20
            height: 45; color: "#885F5F5F"
            border.width: 1; border.color: "#424242"
            radius: 6; smooth: true

            Button {
                text: "Valider"
                textInput { font.pixelSize: 14 }
                width: 75
                height: 25
                border.color: "#424242"
                anchors.right: cancelButton.left
                anchors.top: parent.top
                anchors.topMargin: 5
                anchors.rightMargin: 5
                radius: Math.floor(height/2)
                onClicked: popup.finished(true);
            }

            Button {
                id: cancelButton
                text: "Annuler"
                textInput { font.pixelSize: 14 }
                width: 75
                height: 25
                border.color: "#424242"
                anchors.right: parent.right
                anchors.top: parent.top
                anchors.topMargin: 5
                anchors.rightMargin: 10
                radius: Math.floor(height/2)
                onClicked: popup.finished(false);
            }
        }
    }
}
