/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

import QtQuick 1.0
import Utopital 1.0
import "Components"
import "Utils/logic.js" as JS

Rectangle {
    id: main
    width: 650
    height: 400
    opacity: 0.1

    /*########
    # Visibility
    ########*/

    // Frames
    property bool loginVisible: true
    property bool homeVisible: false
    property bool menusVisible: false

    // Modules
    property bool urgencyVisible: false
    property bool analysisVisible: false
    property bool patientFollowUpVisible: false
    property bool chatVisible: false

    signal loadHomeRequest(bool firstTime)
    signal loadModRequest(int guid)

    onLoginVisibleChanged: {
        if (loginVisible) {
            homeVisible = false;
            menusVisible = false;
            patientFollowUpVisible = false;
            chatVisible = false;
            urgencyVisible = false;
            analysisVisible = false;
        }
    }
    onHomeVisibleChanged: {
        if (homeVisible) {
            loginVisible = false;
            menusVisible = false;
            patientFollowUpVisible = false;
            chatVisible = false;
            urgencyVisible = false;
            analysisVisible = false;
        }
    }
    onLoadHomeRequest: {
        homeVisible = true;
        if (firstTime) {
            anchor.requestForPatientsModel();
        }
    }
    onLoadModRequest: {
        loginVisible = false;
        homeVisible = false;
        menusVisible = false;
        patientFollowUpVisible = false;
        chatVisible = false;
        urgencyVisible = false;
        analysisVisible = false;

        switch (guid) {
            case 0: urgencyVisible = true; break;
            case 1: analysisVisible = true; break;
            case 2: patientFollowUpVisible = true; break;
            case 3: chatVisible = true; break;
            default: homeVisible = true; break;
        }
    }

    /*########
    # Models
    ########*/

    ListModel {
        id: menusModel
        ListElement { name: "Service d'urgence"; image: "qrc:/images/urgence.png" }
        ListElement { name: "Résultats d'analyses"; image: "qrc:/images/analyses.png" }
        ListElement { name: "Suivi des patients"; image: "qrc:/images/patients.png" }
        ListElement { name: "Le salon d'Utopital"; image: "qrc:/images/salon.png" }
    }
    ListModel { id: medicineModel }
    ListModel { id: patientsModel }
    ListModel { id: bodyPartsModel }
    ListModel { id: patientInfos }
    ListModel { id: patientDiagnostics }
    ListModel { id: patientSymptoms }
    ListModel { id: patientPrescriptions }
    ListModel { id: patientUrgenciedModel }
    ListModel { id: chatUsers }
    ListModel { id: scansModel }
    ListModel { id: radiosModel }
    ListModel { id: scanPaths }
    ListModel { id: radioPaths }

    /*########
    # Frames
    ########*/

    Frame_WaitForConnection {
        visible: !anchor.connected
        z: 15

        onVisibleChanged: {
            if (userInChat) {
                anchor.enterChat();
                main.userInChat = true;
            }
        }
    }

    Frame_Login {
        visible: loginVisible
        opacity: visible ? 1 : 0
    }

    Frame_Home {
        id: home
        visible: homeVisible
        opacity: visible ? 1 : 0
    }

    /*########
    # Modules
    ########*/

    Mod_PatientFollowUp {
        id: patientFollowUp
        visible: patientFollowUpVisible
        opacity: visible ? 1 : 0
    }

    Mod_Chat {
        id: chat
        visible: chatVisible
        opacity: visible ? 1 : 0
    }

    Mod_Urgency {
        id: urgency
        visible: urgencyVisible
        opacity: visible ? 1 : 0
    }

    Mod_Analysis {
        id: analysis
        visible: analysisVisible
        opacity: visible ? 1 : 0
    }

    /*########
    # Miscellaneous
    #######*/

    property int patientCurrentlyViewed: -1
    property bool userInChat: false
    signal modifyOrAddPatient(int patientId, string name,
        string birthyear, bool man, string address,
        bool alive, bool canGoHome, bool urgencied);

    Behavior on opacity { NumberAnimation { duration: 500 } }

    Rectangle {
        z: 100; anchors.fill: parent
        anchors.margins: 1
        color: "transparent"
        border.width: 2
        border.color: "grey"
    }

    Component.onCompleted: {
        opacity = 1;
    }

    onModifyOrAddPatient: {
        JS.modifyOrAddPatient(patientId, name, birthyear,
            man, address, alive, canGoHome, urgencied);
    }

    function getScans(index) {
        JS.getScans(index, 0);
    }

    function getRadios(index) {
        JS.getRadios(index, 0, 0);
    }

    Connections {
        target: anchor
        onReceiveMedicineModel: {
            // Réception du JSON contenant les patients
            // puis placement dans le ListModel associé :
            var json = JS.parseJSON(medicine);
            JS.fillMedicineModel(json);
        }
        onReceivePatientsModel: {
            // Réception du JSON contenant les patients
            // puis placement dans le ListModel associé :
            var json = JS.parseJSON(patients);
            JS.fillPatientsModel(json);
        }
        onReceiveBodyPartsModel: {
            // Réception du JSON contenant les parties du corps
            // puis placement dans le ListModel associé :
            var json = JS.parseJSON(parts);
            JS.fillBodyPartsModel(json);
        }
        onReceivePatientInfos: {
            // Réception du JSON contenant les infos du patient
            // puis placement dans le ListModel associé :
            var json = JS.parseJSON(patientInfos);
            JS.fillPatientInfosModel(json);
        }
        onReceiveNewPatientInfos: {
            // Si les modèles n'ont pas encore été remplis, ne fait rien :
            if (patientsModel.count == 0 || patientInfos.count == 0)
                return;
            // Réception du JSON contenant les infos du nouveau patient
            // puis placement dans le ListModel associé :
            var json = JS.parseJSON(patientInfos);
            JS.fillModelWithNewPatient(json);
        }
        onReceivePatientModifs: {
            // Si les modèles n'ont pas encore été remplis, ne fait rien :
            if (patientsModel.count == 0 || patientInfos.count == 0)
                return;
            // Réception du JSON contenant les infos du patient puis
            // placement dans les ListModel associés :
            var json = JS.parseJSON(patientInfos);
            JS.modifyModelsWithPatient(id, json);
            // Ensuite, récupération du nouveau modèle de tous les patients
            // pour gérer le module des urgences (au cas où un patient aurait
            // été envoyé en urgence) :
            anchor.requestForPatientsModel();
        }
        onReceiveNewDiagnostic: {
            // Si les modèles n'ont pas encore été remplis ou si le patient
            // visonné n'est pas le patient dont le dossier a été modifié, ne
            // fait rien :
            if (patientsModel.count == 0 || patientInfos.count == 0
                || patientCurrentlyViewed != id)
                return;
            // Réception du JSON contenant les infos du diagnostic puis
            // placement dans le ListModel associé :
            var json = JS.parseJSON(diagnostic);
            JS.appendNewDiagnostic(json);
        }
        onReceiveNewSymptom: {
            // Si les modèles n'ont pas encore été remplis ou si le patient
            // visonné n'est pas le patient dont le dossier a été modifié, ne
            // fait rien :
            if (patientsModel.count == 0 || patientInfos.count == 0
                || patientCurrentlyViewed != id)
                return;
            // Réception du JSON contenant les infos du symptôme puis
            // placement dans le ListModel associé :
            var json = JS.parseJSON(symptom);
            JS.appendNewSymptom(json);
        }
        onReceiveRemovePrescription: {
            // Si les modèles n'ont pas encore été remplis ou si le patient
            // visonné n'est pas le patient dont le dossier a été modifié, ne
            // fait rien :
            if (patientsModel.count == 0 || patientInfos.count == 0
                || patientCurrentlyViewed != patientId)
                return;
            JS.removePrescription(id);
        }
        onReceiveNewPrescription: {
            // Si les modèles n'ont pas encore été remplis ou si le patient
            // visonné n'est pas le patient dont le dossier a été modifié, ne
            // fait rien :
            if (patientsModel.count == 0 || patientInfos.count == 0
                || patientCurrentlyViewed != patientId)
                return;
            patientFollowUp.loadPatientInfos(patientId);
        }
        onReceiveModifyPrescription: {
            // Si les modèles n'ont pas encore été remplis ou si le patient
            // visonné n'est pas le patient dont le dossier a été modifié, ne
            // fait rien :
            if (patientsModel.count == 0 || patientInfos.count == 0
                || patientCurrentlyViewed != patientId)
                return;
            patientFollowUp.loadPatientInfos(patientId);
        }
        onReceiveEnterChat: {
            JS.memberEnterOrLeaveEvent(username, true);
            anchor.getChatUsers();
        }
        onReceiveLeaveChat: {
            JS.memberEnterOrLeaveEvent(username, false);
            anchor.getChatUsers();
        }
        onReceiveChatMessage: {
            JS.appendMessageInChat(username, message);
        }
        onReceiveChatUsers: {
            JS.setChatUsers(users);
        }
        onReceiveEcgData: {
            urgency.handlePoints(data);
        }
        onReceiveAnalysisModel: {
            // Réception du JSON contenant les résultats d'analyses
            // puis placement dans les ListModel associés :
            var json = JS.parseJSON(analysis);
            JS.fillAnalysisModels(json);
        }
        onReceiveScan: {
            JS.receiveScan(id, part, path);
        }
        onReceiveRadio: {
            JS.receiveRadio(id, x_part, y_part, path);
        }
    }
}
