/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#include "customer.h"
#include "tcpserver.h"

/*!
    \class Customer
    \brief La classe Customer est la classe correspondant à un client.

    Dérivée de QObject, elle correspond à un client connecté au serveur.
    Elle gère la déconnexion du client ainsi que l'arrivée d'un paquet, qu'elle
    vérifie avant de le retransmettre au serveur afin qu'il soit géré par le
    gestionnaire adapté.

    \sa TcpServer
*/

/*!
    \fn Customer::Customer(QTcpSocket *socket, TcpServer *server)
    \brief Constructeur de la classe.

    Construit un objet Customer avec le QTcpSocket \a socket et un pointeur
    vers le TcpServer \a server.
*/

Customer::Customer(QTcpSocket *socket, TcpServer *server)
    : _socket(socket), _server(server), _size(0), _inChat(false)
{
    if (!_socket)
        return;

    connect(_socket, SIGNAL(readyRead()), this, SLOT(read()));
    connect(_socket, SIGNAL(disconnected()), this, SLOT(logout()));
}

/*!
    \fn Customer::~Customer()
    \brief Destructeur de la classe.
*/

Customer::~Customer()
{
    _socket->abort();
    delete _socket;
}

/*!
    \fn QString Customer::getPeerAddress()
    \brief Retourne l'IP du client, ou Unknown si le socket est NULL.
*/

QString Customer::getPeerAddress()
{
    return (_socket) ? _socket->peerAddress().toString() : "Unknown";
}

/*!
    \fn Customer::send(const QByteArray &packet)
    \brief Envoie un paquet au client.
*/

void Customer::send(const QByteArray &packet)
{
    _socket->write(packet);
}

/*!
    \fn Customer::read()
    \brief Fonction de lecture des paquets.

    Récupère progressivement le contenu du paquet envoyé par le client puis
    l'envoit au serveur pour que ce dernier le gère. Cela fait, elle nettoie
    les données restantes pour préparer l'arrivée d'un nouveau paquet.

    \sa TcpServer::handleMessage(QDataStream *data, Customer *customer)
*/

void Customer::read()
{
    QDataStream in(_socket);
    quint64 bytesAvailable = _socket->bytesAvailable();

    if (!_size) {
        if (bytesAvailable < (int)sizeof(quint16))
            return;
        in >> _size;
    }

    if (bytesAvailable < _size)
        return;

    _server->handleMessage(&in, this);
    _socket->readAll();
    _size = 0;
}

/*!
    \fn Customer::logout()
    \brief Gère la déconnexion du client.
*/

void Customer::logout()
{
    _socket->abort();
    _server->handleLogout(this);
}
