/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#include "database.h"
#define DATABASE_FILE "database.sqlite"

Database *Database::_instance = 0;

/*!
    \class Database
    \brief La classe Database est la classe correspondant à la base de données.

    Du fait que l'optique de l'application est de limiter au strict minimum les
    connexions par souci d'optimisation, un système de stockage des données de
    chaque table dans des classes de stockage a été mis en place. Lors de la
    reception d'une demande d'informations de la part d'un client, ce sont ces
    classes qui sont utilisées plutôt qu'un SELECT dans la base de données. Les
    seuls SELECT effectués sont réalisés au lancement de l'application serveur.
*/

/*!
    \fn Database::Database()
    \brief Constructeur de la classe.

    Initialise les classes de stockage.
*/

Database::Database() : AccountStore(ACCOUNT_ENTRY_FIELDS),
    CustomerStore(CUSTOMER_ENTRY_FIELDS),
    CustomerDiagnosticStore(CUSTOMER_DIAGNOSTIC_FIELDS),
    CustomerPrescriptionStore(CUSTOMER_PRESCRIPTION_FIELDS),
    CustomerSymptomStore(CUSTOMER_SYMPTOM_FIELDS),
    CustomerUrgencyStore(CUSTOMER_URGENCY_FIELDS),
    GroupStore(GROUP_ENTRY_FIELDS), InvoiceStore(INVOICE_ENTRY_FIELDS),
    MedicineStore(MEDICINE_ENTRY_FIELDS),
    MedicineLevelStore(MEDICINELEVEL_ENTRY_FIELDS),
    RadioStore(RADIO_ENTRY_FIELDS), ScanStore(SCAN_ENTRY_FIELDS),
    database(NULL)
{
}

/*!
    \fn Database::~Database()
    \brief Destructeur de la classe.

    Vide les classes de stockage, libère la mémoire puis ferme la base de
    données et libère la mémoire qu'elle consommait.
*/

Database::~Database()
{
    AccountStore.clear();
    CustomerStore.clear();
    CustomerDiagnosticStore.clear();
    CustomerPrescriptionStore.clear();
    CustomerSymptomStore.clear();
    CustomerUrgencyStore.clear();
    GroupStore.clear();
    InvoiceStore.clear();
    MedicineStore.clear();
    MedicineLevelStore.clear();
    RadioStore.clear();
    ScanStore.clear();
    database->close();
    delete database;
}

/*!
    \fn void Database::initialize()
    \brief Initialise la base de données et remplit les classes de stockage.
*/

void Database::initialize()
{
    qDebug() << "Initializing database...";

    QString path = QCoreApplication::applicationDirPath()
        + "/" + DATABASE_FILE;

    if (!QFileInfo(path).exists() &&
        !QFile::copy(":/" + QString(DATABASE_FILE), path)) {
        qDebug() << "Could not create database: file can't be copied";
        return;
    }

    if (!database) {
        database = new QSqlDatabase(QSqlDatabase::addDatabase("QSQLITE"));
        database->setDatabaseName(path);
    }

    if (!database->open()) {
        qDebug() << QString("Could not open database: %1")
            .arg(database->lastError().text());
        return;
    }

    AccountStore.load("account", database);
    CustomerStore.load("customer", database);
    CustomerDiagnosticStore.load("customer_diagnostic", database);
    CustomerPrescriptionStore.load("customer_prescription", database);
    CustomerSymptomStore.load("customer_symptom", database);
    CustomerUrgencyStore.load("customer_urgency", database);
    GroupStore.load("group", database);
    InvoiceStore.load("invoice", database);
    MedicineStore.load("medicine", database);
    MedicineLevelStore.load("medicine_level", database);
    RadioStore.load("radio", database);
    ScanStore.load("scan", database);

    qDebug() << "Database successfully initialized.";
}

/*!
    \fn Database::getInstance()
    \brief Permet d'accéder au pointeur de la classe.
*/

Database &Database::getInstance()
{
    if (!_instance)
        _instance = new Database;
    return *_instance;
}

/*!
    \fn Database::freeInstance()
    \brief Détruit le pointeur de la classe.
*/

void Database::freeInstance()
{
    delete _instance;
    _instance = NULL;
}

/*!
    \fn Database::findAccountByName(QString name)
    \brief Retourne l'entrée du compte au nom \a name.
*/

const AccountEntry *Database::findAccountByName(QString name)
{
    int rows = AccountStore.getNumRows();

    if (!rows || name.isEmpty())
        return NULL;

    for (int i = 0; i < rows; i++)
        if (const AccountEntry *entry = AccountStore.getData(i)) {
            const QString &string = entry->name;
            if (string.length() == name.length()
                && name.compare(string, Qt::CaseInsensitive) == 0)
                return entry;
        }

    return NULL;
}
