/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#ifndef DBSTORE_H
#define DBSTORE_H

#include <QtSql>
#include <QString>
#include <QList>

/*!
    \class DBStore
    \brief La classe DBStore permet de stocker le contenu d'une table de la
    base de données.

    Classe template, chacune de ses instances contient l'intégralité d'une
    table de la base de données avec une instance d'une structure dérivée de la
    classe BaseEntry par enregistrement.

    L'intérêt d'avoir des structures de stockage est de ne pas avoir à lancer
    des requêtes SELECT dans la base de données lors d'une demande
    d'informations de la part d'un client, et donc d'optimiser la vitesse de
    communication entre le serveur et les clients.

    \sa BaseEntry, Database
*/

template<typename T>
class DBStore
{
public:

    /*!
        \fn DBStore::DBStore(int num)
        \brief Initialise l'instance de stockage.
    */

    explicit DBStore(int num) : _fieldCount(num), _rowCount(0) { }

    /*!
        \fn DBStore::~DBStore()
        \brief Destructeur de la classe.
    */

    ~DBStore() { }

    /*!
        \fn void DBStore::load(QString table, QSqlDatabase *db)
        \brief Charge le contenu de la \a table de la base de données \a db.

        Le traitement se fait en plusieurs étapes :
        - Une action SELECT pour récupérer le contenu de la table ;
        - Récupération du nombre d'enregistrements selon les propriétés du
        driver ;
        - Replacement au premier enregistrement ;
        - Remplissage de la liste \a _data avec les enregistrements.
    */

    void load(QString table, QSqlDatabase *db) {
        if (!db || !db->isOpen() || table.isEmpty() || !_fieldCount)
            return;

        QSqlQuery query = db->exec("SELECT * FROM `" + table + "`");

        if (db->driver()->hasFeature(QSqlDriver::QuerySize))
            _rowCount = query.size();
        else {
            query.last();
            _rowCount = query.at() + 1;
        }

        query.first();

        if (_rowCount <= 0 || !query.isValid())
            return;

        forever {
            T *data = new T;
            for (int i = 0; i < _fieldCount; i++)
                data->setData(i, query.value(i));
            _data.append(data);
            if (!query.next())
                break;
        }
        return;
    }

    /*!
        \fn void DBStore::addData(T *data)
        \brief Ajoute l'entrée \a data à la liste des entrées.
    */

    void addData(T *data) {
        if (data) {
            _data.append(data);
            _rowCount++;
        }
    }

    /*!
        \fn T * DBStore::getData(int index)
        \brief Retourne l'entrée \a index si existante, sinon NULL.
    */

    T *getData(int index) {
        return (_rowCount <= index)
            ? NULL : _data.at(index);
    }

    /*!
        \fn void DBStore::removeData(int index)
        \brief Retire l'entrée \a index si existante et la détruit.
    */

    void removeData(int index) {
        if (_rowCount <= index)
            return;
        T *data = _data.takeAt(index);
        _rowCount--;
        delete data;
    }

    /*!
        \fn int DBStore::getNumRows()
        \brief Retourne le nombre d'enregistrements de la table.
    */

    int getNumRows() {
        return _rowCount;
    }

    /*!
        \fn int DBStore::getNumFields()
        \brief Retourne le nombre de champs de la table.
    */

    int getNumFields() {
        return _fieldCount;
    }

    /*!
        \fn void DBStore::clear()
        \brief Vide le contenu de la liste \a _data et supprime les entrées.
    */

    void clear() {
        qDeleteAll(_data);
        _data.clear();
    }

private:
    /*! La liste des entrées de la table. */
    QList<T*> _data;
    /*! Le nombre de champs de la table. */
    int _fieldCount;
    /*! Le nombre d'entrées de la table. */
    int _rowCount;
};

#endif // DBSTORE_H
