/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#ifndef DBSTRUCTURE_H
#define DBSTRUCTURE_H

#include <QString>
#include <QVariant>

/*!
    \class BaseEntry
    \brief La classe BaseEntry correspond à la base d'une entrée d'une table de
    la base de données.

    Classe template, elle permet d'initaliser les valeurs de ses dérivées, des
    structures correspondant chacune à une entrée d'une table de la base de
    donées. Ces entrées sont contenues dans par des classes de stockage (des
    instances de la classe DBStore) selon la table à laquelle elles
    appartiennent.

    L'intérêt d'avoir des structures de stockage est de ne pas avoir à lancer
    des requêtes SELECT dans la base de données lors d'une demande
    d'informations de la part d'un client, et donc d'optimiser la vitesse de
    communication entre le serveur et les clients.

    \sa DBStore, Database
*/

template<typename T>
class BaseEntry
{
public:
    /*! Remplit un élément de la structure. */
    void setData(int index, QVariant value) {
        if (int *data = static_cast<T*>(this)->dataInt(index))
            *data = value.toInt();
        else if (QString *data = static_cast<T*>(this)->dataString(index))
            *data = value.toString();
    }
};

enum
{
    ACCOUNT_ENTRY_FIELDS            = 4,
    CUSTOMER_ENTRY_FIELDS           = 7,
    CUSTOMER_DIAGNOSTIC_FIELDS      = 3,
    CUSTOMER_SYMPTOM_FIELDS         = 3,
    CUSTOMER_PRESCRIPTION_FIELDS    = 5,
    CUSTOMER_URGENCY_FIELDS         = 1,
    GROUP_ENTRY_FIELDS              = 2,
    INVOICE_ENTRY_FIELDS            = 4,
    MEDICINE_ENTRY_FIELDS           = 4,
    MEDICINELEVEL_ENTRY_FIELDS      = 2,
    RADIO_ENTRY_FIELDS              = 7,
    SCAN_ENTRY_FIELDS               = 6
};

/*!
    \struct AccountEntry
    \brief La structure AccountEntry contient les données d'une entrée de la
    table account.
    \sa DBStore, Database
*/

struct AccountEntry : public BaseEntry<AccountEntry>
{
    int id;                         /*!< Champ n° 0 */
    QString name;                   /*!< Champ n° 1 */
    QString password;               /*!< Champ n° 2 */
    int level;                      /*!< Champ n° 3 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &id;
            case 3: return &level;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int index) {
        switch (index) {
            case 1: return &name;
            case 2: return &password;
            default: return NULL;
        }
    }
};

/*!
    \struct CustomerEntry
    \brief La structure CustomerEntry contient les données d'une entrée de la
    table customer.
    \sa DBStore, Database
*/

struct CustomerEntry : public BaseEntry<CustomerEntry>
{
    int id;                         /*!< Champ n° 0 */
    QString name;                   /*!< Champ n° 1 */
    int gender;                     /*!< Champ n° 2 */
    int birthyear;                  /*!< Champ n° 3 */
    QString address;                /*!< Champ n° 4 */
    int canGoHome;                  /*!< Champ n° 5 */
    int isDead;                     /*!< Champ n° 6 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &id;
            case 2: return &gender;
            case 3: return &birthyear;
            case 5: return &canGoHome;
            case 6: return &isDead;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int index) {
        switch (index) {
            case 1: return &name;
            case 4: return &address;
            default: return NULL;
        }
    }
};

/*!
    \struct CustomerDiagnosticEntry
    \brief La structure CustomerDiagnosticEntry contient les données d'une
    entrée de la table customer_diagnostic.
    \sa DBStore, Database, CustomerEntry
*/

struct CustomerDiagnosticEntry : public BaseEntry<CustomerDiagnosticEntry>
{
    int customer_id;                /*!< Champ n° 0 */
    int date;                       /*!< Champ n° 1 */
    QString diagnostic;             /*!< Champ n° 2 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &customer_id;
            case 1: return &date;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int index) {
        switch (index) {
            case 2: return &diagnostic;
            default: return NULL;
        }
    }
};

/*!
    \struct CustomerPrescriptionEntry
    \brief La structure CustomerPrescriptionEntry contient les données d'une
    entrée de la table customer_prescription.
    \sa DBStore, Database, CustomerEntry
*/

struct CustomerPrescriptionEntry : public BaseEntry<CustomerPrescriptionEntry>
{
    int id;                         /*!< Champ n° 0 */
    int customer_id;                /*!< Champ n° 1 */
    int medicine_id;                /*!< Champ n° 2 */
    int date;                       /*!< Champ n° 3 */
    QString duration;               /*!< Champ n° 4 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &id;
            case 1: return &customer_id;
            case 2: return &medicine_id;
            case 3: return &date;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int index) {
        switch (index) {
            case 4: return &duration;
            default: return NULL;
        }
    }
};

/*!
    \struct CustomerSymptomEntry
    \brief La structure CustomerSymptomEntry contient les données d'une entrée
    de la table customer_symptom.
    \sa DBStore, Database, CustomerEntry
*/

struct CustomerSymptomEntry : public BaseEntry<CustomerSymptomEntry>
{
    int customer_id;                /*!< Champ n° 0 */
    int date;                       /*!< Champ n° 1 */
    QString symptom;                /*!< Champ n° 2 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &customer_id;
            case 1: return &date;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int index) {
        switch (index) {
            case 2: return &symptom;
            default: return NULL;
        }
    }
};

/*!
    \struct CustomerUrgencyEntry
    \brief La structure CustomerUrgencyEntry contient les données d'une entrée
    de la table customer_urgency.
    \sa DBStore, Database, CustomerEntry
*/

struct CustomerUrgencyEntry : public BaseEntry<CustomerUrgencyEntry>
{
    int customer_id;                /*!< Champ n° 0 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &customer_id;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int /*index*/) {
        return NULL;
    }
};

/*!
    \struct GroupEntry
    \brief La structure GroupEntry contient les données d'une entrée de la
    table group.
    \sa DBStore, Database
*/

struct GroupEntry : public BaseEntry<GroupEntry>
{
    int id;                         /*!< Champ n° 0 */
    QString name;                   /*!< Champ n° 1 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &id;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int index) {
        switch (index) {
            case 1: return &name;
            default: return NULL;
        }
    }
};

/*!
    \struct InvoiceEntry
    \brief La structure InvoiceEntry contient les données d'une entrée de la
    table invoice.
    \sa DBStore, Database
*/

struct InvoiceEntry : public BaseEntry<InvoiceEntry>
{
    int id;                         /*!< Champ n° 0 */
    int customer_id;                /*!< Champ n° 1 */
    int pay_date;                   /*!< Champ n° 2 */
    QString credit_card_number;     /*!< Champ n° 3 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &id;
            case 1: return &customer_id;
            case 2: return &pay_date;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int index) {
        switch (index) {
            case 3: return &credit_card_number;
            default: return NULL;
        }
    }
};

/*!
    \struct MedicineEntry
    \brief La structure MedicineEntry contient les données d'une entrée de la
    table medicine.
    \sa DBStore, Database
*/

struct MedicineEntry : public BaseEntry<MedicineEntry>
{
    int id;                         /*!< Champ n° 0 */
    int level;                      /*!< Champ n° 1 */
    QString name;                   /*!< Champ n° 2 */
    QString description;            /*!< Champ n° 3 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &id;
            case 1: return &level;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int index) {
        switch (index) {
            case 2: return &name;
            case 3: return &description;
            default: return NULL;
        }
    }
};

/*!
    \struct MedicineLevelEntry
    \brief La structure MedicineLevelEntry contient les données d'une entrée de
    la table medicine_level.
    \sa DBStore, Database, MedicineEntry
*/

struct MedicineLevelEntry : public BaseEntry<MedicineLevelEntry>
{
    int id;                         /*!< Champ n° 0 */
    QString name;                   /*!< Champ n° 1 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &id;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int index) {
        switch (index) {
            case 1: return &name;
            default: return NULL;
        }
    }
};

/*!
    \struct RadioEntry
    \brief La structure RadioEntry contient les données d'une entrée de la
    table radio.
    \sa DBStore, Database
*/

struct RadioEntry : public BaseEntry<RadioEntry>
{
    int id;                         /*!< Champ n° 0 */
    int customer_id;                /*!< Champ n° 1 */
    int group_id;                   /*!< Champ n° 2 */
    int x_parts;                    /*!< Champ n° 3 */
    int y_parts;                    /*!< Champ n° 4 */
    QString name;                   /*!< Champ n° 5 */
    QString format;                 /*!< Champ n° 6 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &id;
            case 1: return &customer_id;
            case 2: return &group_id;
            case 3: return &x_parts;
            case 4: return &y_parts;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int index) {
        switch (index) {
            case 5: return &name;
            case 6: return &format;
            default: return NULL;
        }
    }
};

/*!
    \struct ScanEntry
    \brief La structure ScanEntry contient les données d'une entrée de la table
    scan.
    \sa DBStore, Database
*/

struct ScanEntry : public BaseEntry<ScanEntry>
{
    int id;                         /*!< Champ n° 0 */
    int customer_id;                /*!< Champ n° 1 */
    int group_id;                   /*!< Champ n° 2 */
    int parts;                      /*!< Champ n° 3 */
    QString name;                   /*!< Champ n° 4 */
    QString format;                 /*!< Champ n° 5 */

    /*! Accesseur permettant le remplissage des données de la structure. */
    int *dataInt(int index) {
        switch (index) {
            case 0: return &id;
            case 1: return &customer_id;
            case 2: return &group_id;
            case 3: return &parts;
            default: return NULL;
        }
    }
    /*! Accesseur permettant le remplissage des données de la structure. */
    QString *dataString(int index) {
        switch (index) {
            case 4: return &name;
            case 5: return &format;
            default: return NULL;
        }
    }
};

#endif // DBSTRUCTURE_H
