#include "opcodes.h"
#include "simulator.h"
#include "customer.h"

Simulator *Simulator::_instance = 0;

/*!
    \class Simulator
    \brief La classe Simulator est la classe correspondant au simulateur de
    données.

    Du fait que l'application ne soit pas branchée à un quelconque matériel
    médical, il est nécessaire qu'elle possède un simulateur de données lui
    permettant d'envoyer des données aux clients. C'est ainsi que l'ECG est
    gérée.
*/

/*!
    \fn Simulator::Simulator()
    \brief Constructeur de la classe.
*/

Simulator::Simulator() { }

/*!
    \fn Simulator::~Simulator()
    \brief Destructeur de la classe.
*/

Simulator::~Simulator() { }

/*!
    \fn void Simulator::initialize()
    \brief Initialise le simulateur de données avec les données relative aux
    simulation (ici, les ECG).
*/

void Simulator::initialize()
{
    qDebug() << "Initializing simulator...";

    QFile file(":/simulation/ecg/ecg.txt");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "Failed to initialize the simulator.";
        return;
    }

    QTextStream stream(&file);
    while (!stream.atEnd())
        _ecgPoints.append(stream.readLine().toFloat());
    file.close();

    _updateTimer = new QTimer(this);
    _updateTimer->setInterval(UPDATE_INTERVAL);

    connect(_updateTimer, SIGNAL(timeout()), this, SLOT(sendData()));

    qDebug() << "Simulator successfully initialized.";
}

/*!
    \fn Simulator::getInstance()
    \brief Permet d'accéder au pointeur de la classe.
*/

Simulator &Simulator::getInstance()
{
    if (!_instance)
        _instance = new Simulator;
    return *_instance;
}

/*!
    \fn Simulator::freeInstance()
    \brief Détruit le pointeur de la classe.
*/

void Simulator::freeInstance()
{
    delete _instance;
    _instance = NULL;
}

/*!
    \fn Simulator::addClientWForECG(Customer *customer)
    \brief Ajoute le client à la liste des clients en attente des données
    de l'ECG.
*/

void Simulator::addClientWForECG(Customer *customer)
{
    _ecgClients.append(customer);
    _ecgProgress.append(0);
    if (!_updateTimer->isActive())
        _updateTimer->start();
}

/*!
    \fn Simulator::removeClientWForECG(Customer *customer)
    \brief Retire le client de la liste des clients en attente des données
    de l'ECG.
*/

void Simulator::removeClientWForECG(Customer *customer)
{
    int pos = _ecgClients.indexOf(customer);
    if (pos != -1) {
        _ecgClients.removeAt(pos);
        _ecgProgress.removeAt(pos);
        if (_ecgClients.isEmpty())
            _updateTimer->stop();
    }
}

/*!
    \fn Simulator::sendData()
    \brief Fonction d'envoi des données à tous les clients en attente de
    données.

    Cette fonction est appelée toutes les 50 ms et envoit à tous les clients en
    attente des données du simulateur un paquet les contenant.

    L'ECG est réalisée de manière à ce que les courbes ne soient pas toujours
    les mêmes : une notion de "bruit" a été intégrée, gérée très simplement
    avec de l'aléatoire.
*/

void Simulator::sendData()
{
    for (int i = 0; i < _ecgClients.count(); i++) {
        Customer *customer = _ecgClients.at(i);
        int progress = _ecgProgress.at(i);

        if (!customer)
            continue;

        QList<float> list;

        // Used to simulate a real patient (avoid linear curves) :
        float noise = (qrand()%100) / 1000.0f;

        for (int j = 0; j < UPDATE_INTERVAL; j++) {
            progress = (progress >= MAX_ECG_POINTS-1) ? 0 : (progress + 1);
            list.append(_ecgPoints.at(progress) - noise);
        }

        QByteArray packet;
        QDataStream out(&packet, QIODevice::WriteOnly);

        out << (quint64)0;
        out << int(MSG_ECG_DATA);
        out << list;
        out.device()->seek(0);
        out << (quint64)(packet.size() - sizeof(quint64));

        customer->send(packet);

        _ecgProgress[i] = progress;
    }
}
