/*
# Copyright (c) 2011 Louis du Verdier
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

#include "tcpserver.h"
#include "customer.h"
#include "opcodes.h"
#include "database.h"
#include "simulator.h"

/*!
    \class TcpServer
    \brief La classe TcpServer est la classe correspondant au serveur.

    Dérivée de QTcpServer, elle permet de gérer l'intégralité des clients
    connectés à l'application serveur. Elle écoute le port 45354 et gère les
    paquets, identifiables par leur opcode, envoyés par les clients.

    \sa Customer
*/

/*!
    \fn TcpServer::initialize()
    \brief Initialise le serveur.
*/

void TcpServer::initialize()
{
    qDebug() << "Initializing server...";

    if (!listen(QHostAddress::Any, 45354)) {
        qDebug() << QString("Could not listen for incoming "
            "connections on port 45354: %1").arg(errorString());
        return;
    }

    connect(this, SIGNAL(newConnection()), this, SLOT(handleConnection()));

    qDebug() << "Server successfully initialized.";
}

/*!
    \fn TcpServer::prepareJSON(const QString &str)
    \brief Retourne une chaine parsée.

    L'effet est équivalent à celui de la fonction addslashes de PHP, bien que
    seuls les antislashs et les guillemets soient traités.
*/

QString TcpServer::prepareJSON(const QString &str)
{
    QString data = str;
    data.replace("\\", "\\\\");
    data.replace("\"", "\\\"");
    data.replace("\n", "\\n");
    return data;
}

/*!
    \fn TcpServer::fillStrings(QList<QString> list, ...)
    \brief Remplit des QString avec les données de \a list.
*/

void TcpServer::fillStrings(QList<QString> list, ...)
{
    va_list arg;
    va_start(arg, list);
    for (int i = 0; i < list.count(); i++) {
        QString *data = va_arg(arg, QString*);
        if (data)
            *data = list[i];
    }
    va_end(arg);
}

/*!
    \fn TcpServer::handleConnection()
    \brief Gère les nouvelles connexions.

    Lorsqu'un client se connecte au serveur, il est ajouté à la liste des
    clients connectés au serveur pour une utilisation ultérieure.
*/

void TcpServer::handleConnection()
{
    Customer *customer = new Customer(nextPendingConnection(), this);
    _customers.append(customer);

    qDebug() << QString("New connection from %1.")
        .arg(customer->getPeerAddress()).toAscii().constData();
}

/*!
    \fn TcpServer::handleLogout(Customer *customer)
    \brief Gère la déconnexion du client \a customer.

    Retire le client de la liste des clients connectés au serveur puis le
    supprime. Dans le cas où le client serait dans la liste des clients en
    attente des informations en provenance du simulateur, celui-ci est retiré
    de la liste. Si le client était présent dans le chat, notifie les autres
    clients connectés de sa sortie du salon.
*/

void TcpServer::handleLogout(Customer *customer)
{
    _customers.removeOne(customer);

    if (customer->inChat()) {
        QByteArray packet;
        QDataStream out(&packet, QIODevice::WriteOnly);

        out << (quint64)0;
        out << int(MSG_LEAVE_CHAT);
        out << customer->getName();
        out.device()->seek(0);
        out << (quint64)(packet.size() - sizeof(quint64));

        foreach (Customer *c, _customers) {
            if (c->inChat())
                c->send(packet);
        }
    }

    simulator.removeClientWForECG(customer);

    qDebug() << QString("Customer %1 (%2) has been disconnected.")
        .arg(customer->getName(), customer->getPeerAddress())
        .toAscii().constData();

    customer->deleteLater();
    customer = NULL;
}

/*!
    \fn TcpServer::handleMessage(QDataStream *data, Customer *customer)
    \brief Gestionnaire global des paquets.

    Cette fonction est appelée depuis les clients une fois qu'un paquet a été
    entièrement reçu afin de le rediriger vers le gestionnaire de paquets
    associé à l'opcode qu'il possède.

    Dans un premier temps, on récupère l'opcode, puis on vérifie qu'elle est
    bien valide. Enfin, on appelle le gestionnaire associé.
*/

void TcpServer::handleMessage(QDataStream *data, Customer *customer)
{
    int opcode;
    QString username, password;
    *data >> opcode >> username >> password;

    if (opcode >= MSG_MAX_OPCODES) {
        qDebug() << QString("Receive unexpected opcode %1 from %2. Skipped.")
            .arg(opcode).arg(customer->getPeerAddress()).toAscii().constData();
        return;
    }

    if (username != CLIENT_USERNAME || password != CLIENT_PASSWORD) {
        qDebug() << QString("Receive fake packet from %1. Skipped.")
            .arg(customer->getPeerAddress()).toAscii().constData();
        return;
    }

    OpcodeHandler &handle = opcodes[opcode];
    (this->*handle.handler)(data, customer);

    qDebug() << QString("Receive opcode %1 from %2 (%3).")
        .arg(handle.name, customer->getName(), customer->getPeerAddress())
        .toAscii().constData();
}

/*!
    \fn TcpServer::preparePatient(const int &id)
    \brief Retourne la chaine JSON des données du dossier du patient \a id.
*/

QString TcpServer::preparePatient(const int &id) {
    QString json = "{\"data\":";

    if (const CustomerEntry *entry = db.CustomerStore.getData(id)) {
        json += "{";
        json += "\"name\":\"" + prepareJSON(entry->name) + "\",";
        json += "\"gender\":\"" + QString::number(entry->gender) + "\",";
        json += "\"birthyear\":\"" + QString::number(entry->birthyear) + "\",";
        json += "\"adress\":\"" + prepareJSON(entry->address) + "\",";
        json += "\"canGoHome\":\"" + QString::number(entry->canGoHome) + "\",";
        json += "\"isDead\":\"" + QString::number(entry->isDead) + "\",";
        json += "\"diagnostics\":[";

        for (int i = 0; i < db.CustomerDiagnosticStore.getNumRows(); i++) {
            if (const CustomerDiagnosticEntry *diagnostic
                = db.CustomerDiagnosticStore.getData(i)) {
                if (diagnostic->customer_id == id) {
                    json += "{";
                    json += "\"date\":\""
                        + QString::number(diagnostic->date) + "\",";
                    json += "\"diagnostic\":\""
                        + prepareJSON(diagnostic->diagnostic) + "\"";
                    json += "},";
                }
            }
        }

        if (json.right(1) == ",")
            json.chop(1);
        json += "],";
        json += "\"symptoms\":[";

        for (int i = 0; i < db.CustomerSymptomStore.getNumRows(); i++) {
            if (const CustomerSymptomEntry *symptom
                = db.CustomerSymptomStore.getData(i)) {
                if (symptom->customer_id == id) {
                    json += "{";
                    json += "\"date\":\""
                        + QString::number(symptom->date) + "\",";
                    json += "\"symptom\":\""
                        + prepareJSON(symptom->symptom) + "\"";
                    json += "},";
                }
            }
        }

        if (json.right(1) == ",")
            json.chop(1);
        json += "],";

        json += "\"prescriptions\":[";

        for (int i = 0; i < db.CustomerPrescriptionStore.getNumRows(); i++) {
            if (const CustomerPrescriptionEntry *prescription
                = db.CustomerPrescriptionStore.getData(i)) {
                if (prescription->customer_id == id) {
                    json += "{";
                    json += "\"prescription_id\":\""
                        + QString::number(prescription->id) + "\",",
                    json += "\"medicine_id\":\""
                        + QString::number(prescription->medicine_id) + "\",";
                    json += "\"date\":\""
                        + QString::number(prescription->date) + "\",";
                    json += "\"duration\":\""
                        + prepareJSON(prescription->duration) + "\"";
                    json += "},";
                }
            }
        }

        if (json.right(1) == ",")
            json.chop(1);
        json += "],";

        bool urgencied = false;

        for (int i = 0; i < db.CustomerUrgencyStore.getNumRows(); i++) {
            if (const CustomerUrgencyEntry *urgency
                = db.CustomerUrgencyStore.getData(i)) {
                if (urgency->customer_id == id) {
                    urgencied = true;
                    break;
                }
            }
        }

        json += "\"urgencied\":\"" + QString::number(urgencied) + "\"";
        json += "}";
    }

    json += "}";

    return json;
}

/*!
    \fn TcpServer::unhandled(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets inutilisés.

    Ne fait rien.
*/

void TcpServer::unhandled(QDataStream *data, Customer *customer)
{
    Q_UNUSED(data);
    Q_UNUSED(customer);
}

/*!
    \fn TcpServer::handlePing(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets correspondant aux pings.

    Envoit un pong au client ayant envoyé un ping.
*/

void TcpServer::handlePing(QDataStream *data, Customer *customer)
{
    Q_UNUSED(data);

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_PONG);
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    customer->send(packet);
}

/*!
    \fn TcpServer::handleLogin(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets correspondant aux tentatives de login.

    Envoit true si les identifiants passés sont corrects, sinon false.
*/

void TcpServer::handleLogin(QDataStream *data, Customer *customer)
{
    QString username, password;
    bool answer = false;

    *data >> username;
    *data >> password;

    if (const AccountEntry *entry = db.findAccountByName(username))
    {
        const QString &str = QCryptographicHash::hash(password.toAscii(),
            QCryptographicHash::Sha1).toHex();
        if (str == entry->password) {
            answer = true;
            customer->setName(username);
        }
    }

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_LOGIN);
    out << answer;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    customer->send(packet);
}

/*!
    \fn TcpServer::handleMedicineModel(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets correspondant aux demandes de la liste des
    médicaments disponibles.

    Envoit une chaine de caractères au format JSON contenant les médicaments.
*/

void TcpServer::handleMedicineModel(QDataStream *data, Customer *customer)
{
    Q_UNUSED(data);
    QString json = "{\"data\":[";
    QStringList medicineLevel;

    for (int i = 0; i < db.MedicineLevelStore.getNumRows(); i++)
        if (const MedicineLevelEntry *entry = db.MedicineLevelStore.getData(i))
            medicineLevel.append(entry->name);

    for (int i = 0; i < db.MedicineStore.getNumRows(); i++)
        if (const MedicineEntry *entry = db.MedicineStore.getData(i)) {
            json += "{";
            json += "\"name\":\"" + prepareJSON(entry->name) + "\",";
            json += "\"description\":\""
                + prepareJSON(entry->description) + "\",";
            json += "\"level\":\""
                + prepareJSON(medicineLevel.count() > (entry->level-1)
                ? medicineLevel[entry->level-1] : "") + "\"";
            json += "},";
        }

    json.chop(1);
    json += "]}";

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_QUERY_MEDICINE_MODEL);
    out << json;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    customer->send(packet);
}

/*!
    \fn TcpServer::handlePatientsModel(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant le modèle contenant les patients.

    Envoit une chaine de caractères au format JSON contenant les patients.
*/

void TcpServer::handlePatientsModel(QDataStream *data, Customer *customer)
{
    Q_UNUSED(data);
    QString json = "{\"data\":[";

    int rows = db.CustomerStore.getNumRows();

    for (int i = 0; i < rows; i++)
        if (const CustomerEntry *entry = db.CustomerStore.getData(i)) {
            json += "{";
            json += "\"name\":\"" + prepareJSON(entry->name) + "\",";
            json += "\"gender\":\"" + QString::number(entry->gender) + "\",";
            json += "\"birthyear\":\""
                + QString::number(entry->birthyear) + "\",";
            json += "\"adress\":\"" + prepareJSON(entry->address) + "\",";
            json += "\"canGoHome\":\""
                + QString::number(entry->canGoHome) + "\",";
            json += "\"isDead\":\"" + QString::number(entry->isDead) + "\",";
            bool urgencied = false;
            for (int i = 0; i < db.CustomerUrgencyStore.getNumRows(); i++) {
                if (const CustomerUrgencyEntry *urgency
                    = db.CustomerUrgencyStore.getData(i)) {
                    if (urgency->customer_id == entry->id) {
                        urgencied = true;
                        break;
                    }
                }
            }
            json += "\"urgencied\":\"" + QString::number(urgencied) + "\"";
            json += "},";
        }

    json.chop(1);
    json += "]}";

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_QUERY_PATIENTS_MODEL);
    out << json;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    customer->send(packet);
}

/*!
    \fn TcpServer::handleBodyPartsModel(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant le modèle contenant les noms et
    ids des différentes parties du corps humain.

    Envoit une chaine de caractères au format JSON contenant les données des
    parties du corps humain.
*/

void TcpServer::handleBodyPartsModel(QDataStream *data, Customer *customer)
{
    Q_UNUSED(data);
    QString json = "{\"data\":[";

    int rows = db.GroupStore.getNumRows();

    for (int i = 0; i < rows; i++)
        if (const GroupEntry *entry = db.GroupStore.getData(i)) {
            json += "{";
            json += "\"id\":\"" + QString::number(entry->id) + "\",";
            json += "\"name\":\"" + prepareJSON(entry->name) + "\"";
            json += "},";
        }

    json.chop(1);
    json += "]}";

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_QUERY_BODY_PARTS);
    out << json;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    customer->send(packet);
}

/*!
    \fn TcpServer::handlePatientQuery(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant les informations d'un patient.

    Envoit une chaine de caractères au format JSON contenant les informations
    concernant le patient.
*/

void TcpServer::handlePatientQuery(QDataStream *data, Customer *customer)
{
    int id;
    *data >> id;

    QString json = preparePatient(id);

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_QUERY_PATIENT);
    out << json;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    customer->send(packet);
}

/*!
    \fn TcpServer::handleNewPatient(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant l'ajout d'un patient.

    Ajoute le nouveau patient puis envoit à tous les clients connectés une
    chaine de caractères au format JSON contenant les informations concernant
    le nouveau patient.
*/

void TcpServer::handleNewPatient(QDataStream *data, Customer *customer)
{
    Q_UNUSED(customer);

    QList<QString> list;
    *data >> list;

    int id = db.CustomerStore.getNumRows();

    if (list.isEmpty())
        return;

    QString name, age, gender, address, dead, home, urgencied, birthyear;
    fillStrings(list, &name, &age, &gender, &address,
        &dead, &home, &urgencied);

    birthyear = QString::number(QDate::currentDate().year() - age.toInt());

    QSqlQuery query(*db.database);
    query.prepare("INSERT INTO customer (id, name, gender, birthyear, "
        "address, canGoHome, isDead) VALUES (:id, :name, :gender, "
        ":birthyear, :address, :canGoHome, :isDead);");
    query.bindValue(":id", id);
    query.bindValue(":name", name);
    query.bindValue(":gender", gender);
    query.bindValue(":birthyear", birthyear);
    query.bindValue(":address", address);
    query.bindValue(":canGoHome", home);
    query.bindValue(":isDead", dead);
    query.exec();

    CustomerEntry *customerEntry = new CustomerEntry();
    customerEntry->id = id;
    customerEntry->address = address;
    customerEntry->birthyear = birthyear.toInt();
    customerEntry->canGoHome = home.toInt();
    customerEntry->gender = gender.toInt();
    customerEntry->isDead = dead.toInt();
    customerEntry->name = name;
    db.CustomerStore.addData(customerEntry);

    if (urgencied == "1" && id >= 0) {
        query.prepare(QString("INSERT INTO customer_urgency "
            "VALUES (%1);").arg(id));
        query.exec();
        CustomerUrgencyEntry *urgencyEntry = new CustomerUrgencyEntry();
        urgencyEntry->customer_id = id;
        db.CustomerUrgencyStore.addData(urgencyEntry);
    }

    QString json = preparePatient(id);

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_NEW_PATIENT);
    out << json;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    foreach (Customer *c, _customers)
        c->send(packet);
}

/*!
    \fn TcpServer::handleModifyPatient(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant la modification d'un patient.

    Modifie les données du patient puis envoit à tous les clients connectés une
    chaine de caractères au format JSON contenant les informations concernant
    le dossier modifié du patient.
*/

void TcpServer::handleModifyPatient(QDataStream *data, Customer *customer)
{
    Q_UNUSED(customer);

    int id;
    QList<QString> list;
    *data >> id >> list;

    if (list.isEmpty())
        return;

    QString name, age, gender, address, dead, home, urgencied, birthyear;
    fillStrings(list, &name, &age, &gender, &address,
        &dead, &home, &urgencied);

    birthyear = QString::number(QDate::currentDate().year() - age.toInt());

    QSqlQuery query(*db.database);
    query.prepare("UPDATE customer SET name = :name, gender = :gender, "
        "birthyear = :birthyear, address = :address, canGoHome = :canGoHome, "
        "isDead = :isDead WHERE id = :id");
    query.bindValue(":id", id);
    query.bindValue(":name", name);
    query.bindValue(":gender", gender);
    query.bindValue(":birthyear", birthyear);
    query.bindValue(":address", address);
    query.bindValue(":canGoHome", home);
    query.bindValue(":isDead", dead);
    query.exec();

    if (CustomerEntry *customerEntry = db.CustomerStore.getData(id)) {
        customerEntry->address = address;
        customerEntry->birthyear = birthyear.toInt();
        customerEntry->canGoHome = home.toInt();
        customerEntry->gender = gender.toInt();
        customerEntry->isDead = dead.toInt();
        customerEntry->name = name;
    }

    query.prepare("DELETE FROM customer_urgency WHERE id = :id");
    query.exec();

    for (int i = 0; i < db.CustomerUrgencyStore.getNumRows(); i++) {
        if (const CustomerUrgencyEntry *urgency
            = db.CustomerUrgencyStore.getData(i)) {
            if (urgency->customer_id == id) {
                db.CustomerUrgencyStore.removeData(i);
                break;
            }
        }
    }

    if (urgencied == "1" && id >= 0) {
        query.prepare("INSERT OR IGNORE INTO customer_urgency VALUES :id");
        query.exec();

        CustomerUrgencyEntry *urgencyEntry = new CustomerUrgencyEntry();
        urgencyEntry->customer_id = id;
        db.CustomerUrgencyStore.addData(urgencyEntry);
    }

    QString json = preparePatient(id);

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_MODIFY_PATIENT);
    out << id;
    out << json;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    foreach (Customer *c, _customers)
        c->send(packet);
}

/*!
    \fn TcpServer::handleNewDiagnostic(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant l'ajout d'un nouveau diagnostic
    au dossier d'un patient.

    Ajoute le nouveau diagnostic puis envoit à tous les clients connectés une
    chaine de caractères au format JSON contenant les informations du nouveau
    diagnostic.
*/

void TcpServer::handleNewDiagnostic(QDataStream *data, Customer *customer)
{
    Q_UNUSED(customer);

    int id;
    QString text;
    *data >> id >> text;

    if (text.isEmpty())
        return;

    int timestamp = qRound64(QDateTime::currentMSecsSinceEpoch()/1000);

    QSqlQuery query(*db.database);
    query.prepare("INSERT INTO customer_diagnostic "
        "(customer_id, date, diagnostic) VALUES "
        "(:id, :timestamp, :text);");
    query.bindValue(":id", id);
    query.bindValue(":timestamp", timestamp);
    query.bindValue(":text", text);
    query.exec();

    CustomerDiagnosticEntry *entry = new CustomerDiagnosticEntry();
    entry->customer_id = id;
    entry->date = timestamp;
    entry->diagnostic = text;
    db.CustomerDiagnosticStore.addData(entry);

    QString json = "{\"diagnostic\":{";
    json += "\"date\":\"" + QString::number(timestamp) + "\",";
    json += "\"diagnostic\":\"" + prepareJSON(text) + "\"";
    json += "}}";

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_NEW_DIAGNOSTIC);
    out << id;
    out << json;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    foreach (Customer *c, _customers)
        c->send(packet);
}

/*!
    \fn TcpServer::handleNewSymptom(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant l'ajout d'un nouveau symptôme au
    dossier d'un patient.

    Ajoute le nouveau symptôme puis envoit à tous les clients connectés une
    chaine de caractères au format JSON contenant les informations du nouveau
    symptôme.
*/

void TcpServer::handleNewSymptom(QDataStream *data, Customer *customer)
{
    Q_UNUSED(customer);

    int id;
    QString text;
    *data >> id >> text;

    if (text.isEmpty())
        return;

    int timestamp = qRound64(QDateTime::currentMSecsSinceEpoch()/1000);

    QSqlQuery query(*db.database);
    query.prepare("INSERT INTO customer_symptom "
        "(customer_id, date, symptom) VALUES "
        "(:id, :timestamp, :text);");
    query.bindValue(":id", id);
    query.bindValue(":timestamp", timestamp);
    query.bindValue(":text", text);
    query.exec();

    CustomerSymptomEntry *entry = new CustomerSymptomEntry();
    entry->customer_id = id;
    entry->date = timestamp;
    entry->symptom = text;
    db.CustomerSymptomStore.addData(entry);

    QString json = "{\"symptom\":{";
    json += "\"date\":\"" + QString::number(timestamp) + "\",";
    json += "\"symptom\":\"" + prepareJSON(text) + "\"";
    json += "}}";

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_NEW_SYMPTOM);
    out << id;
    out << json;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    foreach (Customer *c, _customers)
        c->send(packet);
}

/*!
    \fn TcpServer::handleRemovePrescription(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant la suppression d'une prescription
    du dossier d'un patient.

    Supprime la prescription puis envoit à tous les clients connectés une
    notification comme quoi les prescriptions du patient ont été modifiées.
*/

void TcpServer::handleRemovePrescription(QDataStream *data, Customer *customer)
{
    Q_UNUSED(customer);

    int patientId, id;
    *data >> patientId >> id;

    QSqlQuery query(*db.database);
    query.prepare("DELETE FROM customer_prescription "
        "WHERE id = :id AND customer_id = :patientId");
    query.bindValue(":id", id);
    query.bindValue(":patientId", patientId);
    query.exec();

    for (int i = 0; i < db.CustomerPrescriptionStore.getNumRows(); i++) {
        if (const CustomerPrescriptionEntry *entry
            = db.CustomerPrescriptionStore.getData(i)) {
            if (entry->customer_id == patientId
                && entry->id == id) {
                db.CustomerPrescriptionStore.removeData(i);
                break;
            }
        }
    }

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_REMOVE_PRESCRIPTION);
    out << patientId;
    out << id;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    foreach (Customer *c, _customers)
        c->send(packet);
}

/*!
    \fn TcpServer::handleModifyPrescription(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant la modification d'une
    prescription du dossier d'un patient.

    Modifie la prescription puis envoit à tous les clients connectés une
    notification comme quoi les prescriptions du patient ont été modifiées.
*/

void TcpServer::handleModifyPrescription(QDataStream *data, Customer *customer)
{
    Q_UNUSED(customer);

    int patientId, id, date, medicineId;
    QString duration;
    *data >> patientId >> id >> duration >> date >> medicineId;

    QSqlQuery query(*db.database);
    query.prepare("UPDATE customer_prescription SET "
        "medicine_id = :medicineId, date = :date, duration = :duration "
        "WHERE id = :id AND customer_id = :patientId");
    query.bindValue(":patientId", patientId);
    query.bindValue(":id", id);
    query.bindValue(":medicineId", medicineId);
    query.bindValue(":date", date);
    query.bindValue(":duration", duration);
    query.exec();

    for (int i = 0; i < db.CustomerPrescriptionStore.getNumRows(); i++) {
        if (CustomerPrescriptionEntry *entry
            = db.CustomerPrescriptionStore.getData(i)) {
            if (entry->customer_id == patientId
                && entry->id == id) {
                entry->date = date;
                entry->duration = duration;
                entry->medicine_id = medicineId;
                break;
            }
        }
    }

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_MODIFY_PRESCRIPTION);
    out << patientId;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    foreach (Customer *c, _customers)
        c->send(packet);
}

/*!
    \fn TcpServer::handleNewPrescription(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant la modification d'une
    prescription du dossier d'un patient.

    Modifie la prescription puis envoit à tous les clients connectés une
    notification comme quoi les prescriptions du patient ont été modifiées.
*/

void TcpServer::handleNewPrescription(QDataStream *data, Customer *customer)
{
    Q_UNUSED(customer);

    int patientId, date, medicineId;
    QString duration;
    *data >> patientId >> duration >> date >> medicineId;

    int id = db.CustomerPrescriptionStore.getNumRows();

    QSqlQuery query(*db.database);
    query.prepare("INSERT INTO customer_prescription "
        "(id, customer_id, medicine_id, date, duration) VALUES "
        "(:id, :patientId, :medicineId, :date, :duration);");
    query.bindValue(":patientId", patientId);
    query.bindValue(":id", id);
    query.bindValue(":medicineId", medicineId);
    query.bindValue(":date", date);
    query.bindValue(":duration", duration);
    query.exec();

    CustomerPrescriptionEntry *entry = new CustomerPrescriptionEntry();
    entry->customer_id = patientId;
    entry->date = date;
    entry->duration = duration;
    entry->id = id;
    entry->medicine_id = medicineId;
    db.CustomerPrescriptionStore.addData(entry);

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_MODIFY_PRESCRIPTION);
    out << patientId;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    foreach (Customer *c, _customers)
        c->send(packet);
}

/*!
    \fn TcpServer::handleEnterChat(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets notifiant d'une entrée dans le salon.

    Ajoute le client à la liste des clients présents dans le salon puis envoit
    à tous les clients connectés de la liste une notification de l'entrée.
*/

void TcpServer::handleEnterChat(QDataStream *data, Customer *customer)
{
    Q_UNUSED(data);
    customer->setInChat(true);

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_ENTER_CHAT);
    out << customer->getName();
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    foreach (Customer *c, _customers) {
        if (c->inChat())
            c->send(packet);
    }
}

/*!
    \fn TcpServer::handleLeaveChat(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets notifiant d'une sortie du salon.

    Retire le client de la liste des clients présents dans le salon puis envoit
    à tous les clients connectés de la liste une notification de la sortie.
*/

void TcpServer::handleLeaveChat(QDataStream *data, Customer *customer)
{
    Q_UNUSED(data);
    customer->setInChat(false);

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_LEAVE_CHAT);
    out << customer->getName();
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    foreach (Customer *c, _customers) {
        if (c->inChat())
            c->send(packet);
    }
}

/*!
    \fn TcpServer::handleChatMessage(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets notifiant d'un message dans le salon.

    Envoit à tous les clients de la liste des clients présents dans le salon
    une notification de nouveau message.
*/

void TcpServer::handleChatMessage(QDataStream *data, Customer *customer)
{
    QString message;
    *data >> message;

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_SEND_CHAT_MESSAGE);
    out << customer->getName();
    out << message;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    foreach (Customer *c, _customers) {
        if (c->inChat())
            c->send(packet);
    }
}

/*!
    \fn TcpServer::handleChatUsersRequest(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant la liste des utilisateurs
    connectés au salon.

    Envoit au client la liste des clients connectés au salon.
*/

void TcpServer::handleChatUsersRequest(QDataStream *data, Customer *customer)
{
    Q_UNUSED(data);

    QStringList list;

    foreach (Customer *c, _customers) {
        if (c->inChat())
            list.append(c->getName());
    }

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_QUERY_CHAT_USERS);
    out << list;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    customer->send(packet);
}

/*!
    \fn TcpServer::handleStartECG(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant le lancement de l'ECG.

    Ajoute le client à la liste des clients en attente des données de l'ECG,
    fournies par le simulateur de données.
*/

void TcpServer::handleStartECG(QDataStream *data, Customer *customer)
{
    Q_UNUSED(data);
    simulator.addClientWForECG(customer);
}

/*!
    \fn TcpServer::handleStopECG(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant l'arrêt de l'ECG.

    Retire le client de la liste des clients en attente des données de l'ECG,
    fournies par le simulateur de données.
*/

void TcpServer::handleStopECG(QDataStream *data, Customer *customer)
{
    Q_UNUSED(data);
    simulator.removeClientWForECG(customer);
}

/*!
    \fn TcpServer::handleAnalysisModel(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant le modèle des analyses (scans et
    radios) d'un patient.

    Envoit une chaine de caractères au format JSON contenant les données des
    analyses au client.
*/

void TcpServer::handleAnalysisModel(QDataStream *data, Customer *customer)
{
    Q_UNUSED(data);

    QString json = "{\"data\":{";
    json += "\"radios\":[";

    for (int i = 0; i < db.RadioStore.getNumRows(); i++) {
        if (const RadioEntry *radio = db.RadioStore.getData(i)) {
            json += "{";
            json += "\"id\":\"" + QString::number(radio->id) + "\",";
            json += "\"customer_id\":\""
                + QString::number(radio->customer_id) + "\",";
            json += "\"group_id\":\"" + QString::number(radio->group_id) + "\",";
            json += "\"x_parts\":\"" + QString::number(radio->x_parts) + "\",";
            json += "\"y_parts\":\"" + QString::number(radio->x_parts) + "\"";
            json += "},";
        }
    }

    if (json.right(1) == ",")
        json.chop(1);
    json += "],";

    json += "\"scans\":[";

    for (int i = 0; i < db.ScanStore.getNumRows(); i++) {
        if (const ScanEntry *scan = db.ScanStore.getData(i)) {
            json += "{";
            json += "\"id\":\"" + QString::number(scan->id) + "\",";
            json += "\"customer_id\":\""
                + QString::number(scan->customer_id) + "\",";
            json += "\"group_id\":\"" + QString::number(scan->group_id) + "\",";
            json += "\"parts\":\"" + QString::number(scan->parts) + "\"";
            json += "},";
        }
    }

    if (json.right(1) == ",")
        json.chop(1);
    json += "]}}";

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_QUERY_ANALYSIS_MODEL);
    out << json;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    customer->send(packet);
}

/*!
    \fn TcpServer::handleScanRequest(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant l'image d'une partie d'un scan.

    Envoit l'image de la partie du scan au client.
*/

void TcpServer::handleScanRequest(QDataStream *data, Customer *customer)
{
    QImage ret;
    int id, part;
    *data >> id >> part;

    if (const ScanEntry *entry = db.ScanStore.getData(id)) {
        QString path = ":/simulation/scan/" + entry->name
            + "_" + QString::number(part) + ".jpg";
        if (QFileInfo(path).exists())
            ret.load(path);
        else
            qDebug() << path;
    }

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_QUERY_SCAN);
    out << id;
    out << part;
    out << ret;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    customer->send(packet);
}

/*!
    \fn TcpServer::handleRadioRequest(QDataStream *data, Customer *customer)
    \brief Gestionnaire des paquets demandant l'image d'une partie d'une radio.

    Envoit l'image de la partie de la radio au client.
*/

void TcpServer::handleRadioRequest(QDataStream *data, Customer *customer)
{
    QImage ret;
    int id, x, y;
    *data >> id >> x >> y;

    if (const RadioEntry *entry = db.RadioStore.getData(id)) {
        QString path = ":/simulation/radio/" + entry->name
            + "_" + QString::number(x) + "_" + QString::number(y) + ".jpg";
        if (QFileInfo(path).exists())
            ret.load(path);
        else
            qDebug() << path;
    }

    QByteArray packet;
    QDataStream out(&packet, QIODevice::WriteOnly);

    out << (quint64)0;
    out << int(MSG_QUERY_RADIO);
    out << id;
    out << x;
    out << y;
    out << ret;
    out.device()->seek(0);
    out << (quint64)(packet.size() - sizeof(quint64));

    customer->send(packet);
}
